#include "magnet.h"
#include <cmath>
/*
 * class Field::Magnet
 * implementation of common parts
 *
 * $Author: claudio $
 *
 * $Revision: 1.10 $
 *
 * $Log: magnet.cpp,v $
 * Revision 1.10  2010/01/15 16:30:47  claudio
 * FocussingSignConvention: MAD (K>0) or Vicky (K<0) for quads and sexts.
 *
 * Revision 1.9  2009/06/11 15:54:50  claudio
 * added empirical strength correction factor
 *
 * Revision 1.8  2009/06/11 13:56:07  claudio
 * new formula for strength calculation, electron rest mass is considered for momentum calculation
 *
 * Revision 1.7  2009/01/05 13:16:06  claudio
 * *** empty log message ***
 *
 * Revision 1.6  2007/07/23 08:18:06  claudio
 * exposed "focussing" class variable via  bool getFocussing()
 *
 * Revision 1.5  2007/05/18 12:36:41  claudio
 * refactored for sharing more internal variables and, later, code
 *
 * Revision 1.4  2007/03/28 14:55:37  claudio
 * refactor towards unification of common code
 *
 * Revision 1.3  2007/03/28 13:19:55  claudio
 * development
 *
 * Revision 1.2  2007/03/26 15:04:28  claudio
 * development step of end March
 *
 * Revision 1.1.1.1  2007/03/14 10:27:13  claudio
 * Hlf field module
 *
 */

namespace magnetfield{




/*
 * calculate the electron longitudinal momentum given the total electron energy
 * @param total electron energy [GeV]
 * @return electron momentum [GeV/c]
 */
double p_gev(double en){
	return sqrt(en*en - E_rest_gev_squared);
}

Magnet::Magnet():_focussing(false),_positive(true),_principal(0),_inverse(0),_length(0),_geomLength(0.0),_name("null"),_psName("null"),_minExcitation(NAN),_maxExcitation(NAN),_fieldType(f_dipole), _alpha(1.0){}

Magnet::~Magnet( ) { }


void Magnet::setName( std::string newName )
{
	_name = newName;
}


void Magnet::setSurveyError( surveyError newSurveyError )
{
	_surveyError = newSurveyError;
}


surveyError Magnet::getSurveyError( ) const
{
	return _surveyError;
}


void Magnet::setPsName( std::string newPsName )
{
	_psName = newPsName;
}



std::string Magnet::getName( ) const
{
	return _name;
}

double Magnet::getGeometricLength( ) const
{
	return _geomLength;
}

std::string Magnet::getPsName( ) const
{
	return _psName;
}

void Magnet::setFieldType( magneticFieldType newFieldType )
{
	_fieldType = newFieldType;
}

magneticFieldType Magnet::getFieldType( ) const
{
	return _fieldType;
}

void Magnet::setGeometricLength( const double newLength )
{
	_geomLength=newLength;
}

void Magnet::_updateRange()
{
	double dmin,dmax;
	_principal->get_range(dmin,dmax);
	_minExcitation=dmin;
	_maxExcitation=dmax;
}

bool Magnet::isFocussing() const
{
	return _focussing;
}

bool Magnet::isPositive() const
{
	return _positive;
}

double Magnet::getAlpha() const{
	return _alpha;
}

void Magnet::setAlpha(const double al)
{
	_alpha=al;
	return;
}


/*
 * read the actual global sign convention
 */


focSignConvention Magnet::getFocSignConvetion()
{
	return _foc_sign_convention;
}

/*
 * set the global sign convetion
 */

void Magnet::setFocSignConvention(focSignConvention f_s_c)
{
	_foc_sign_convention=f_s_c;
	return;
}

} //close namespace Field
