/*
 * implementation of Magnetfactory class
 *
 * $Author: claudio $
 *
 * $Revision: 1.23 $
 *
 * $Log: magnetfactory.cpp,v $
 * Revision 1.23  2018-11-15 15:23:42  claudio
 * start release_8 developement
 *
 * Revision 1.22  2014-04-16 11:46:04  claudio
 * corrected bug of dipole building method
 *
 * Revision 1.21  2011/04/26 11:51:23  claudio
 * added mysql_thread_end()  when closing db connection
 *
 * Revision 1.20  2010/11/11 09:43:16  claudio
 * *** empty log message ***
 *
 * Revision 1.19  2009/03/31 07:54:45  claudio
 * mysql_library handling for memory leak prevention
 *
 * Revision 1.18  2009/03/24 09:08:38  claudio
 * create solenoid
 *
 * Revision 1.17  2009/01/07 08:42:57  claudio
 * adapted to new class names for interpolators
 *
 * Revision 1.16  2008/01/14 12:47:01  claudio
 * *** empty log message ***
 *
 * Revision 1.15  2007/12/03 11:38:08  claudio
 * new build layout
 * bug in spline array handling
 *
 * Revision 1.14  2007/11/30 14:11:19  claudio
 * Workaround for unkown BUG
 * TO BE corrected
 *
 * Revision 1.13  2007/09/25 08:16:14  claudio
 * corrected bug in dipole with polynomial energy calibration
 *
 * Revision 1.12  2007/09/04 17:04:09  claudio
 * bug in select for polynomial calibration
 * details in exception msg
 *
 * Revision 1.11  2007/08/06 13:45:29  claudio
 * update type to unsigned int
 *
 * Revision 1.10  2007/07/27 08:18:53  claudio
 * steerer no used anymore: use kicker, corrh,corrv
 *
 * Revision 1.9  2007/07/16 08:21:18  claudio
 * added support of MAGNETDB environemnt variable to pass
 * usr, password and host for db connection.
 *
 * Revision 1.8  2007/07/13 10:48:29  claudio
 * modified SQL query for getting kicker calibrations.
 * N.B.: required upgrade of database schema
 *
 * Revision 1.7  2007/07/10 07:26:56  claudio
 * corrected wrong SQL query
 *
 * Revision 1.6  2007/05/18 12:36:41  claudio
 * refactored for sharing more internal variables and, later, code
 *
 * Revision 1.5  2007/03/29 15:54:29  claudio
 * added creation-handling of kickers
 *
 * Revision 1.4  2007/03/29 12:56:34  claudio
 * added creation-handling of sextupoles
 *
 * Revision 1.3  2007/03/28 13:19:55  claudio
 * development
 *
 * Revision 1.2  2007/03/26 15:04:28  claudio
 * development step of end March
 *
 * Revision 1.1.1.1  2007/03/14 10:27:13  claudio
 * Hlf field module
 *
 */

#include "magnetfactory.h"
#include <sstream>
#include <cstdlib>
#include <string>
#include <sstream>
#include <spline.h>
#include <multipolynomial.h>

#include <iostream>
//TODO: routine to check db coherency and completeness
//TODO: check to have at least 2 values before build spline Interpolator::Interpolator
//TODO: psname!!!!!
using namespace Interpolator;

namespace magnetfield{
void Magnetfactory::initdb(const char* dbname, const char* dbuser, const char* dbpasswd, const char* dbhost)
{
	//init the db connection
	//const char* database="fermi_magnets";
	mysql_library_init(0,0,0);
	mysql_init(&mysql);

	mysql_options(&mysql,MYSQL_READ_DEFAULT_GROUP,"client");


	//C. Scafuri: auto reconnection has been off since 5.0.3. From 5.0.13 it is possible to set it as an option
	// with reconnection enabled DataBase keeps working after timeouts and mysql shutdown/restart
	if(mysql_get_client_version() >= 50013)
	{
		bool my_auto_reconnect=1;
		if(mysql_options(&mysql,MYSQL_OPT_RECONNECT,&my_auto_reconnect) !=0)
			throw std::invalid_argument(mysql_error(&mysql));
	}

	if(! mysql_real_connect(&mysql,dbhost,dbuser,dbpasswd,dbname,0,0,0)){
		throw std::invalid_argument(mysql_error(&mysql));
	}
}

Magnetfactory::Magnetfactory()
{
	std::string env,db,user,passw,host;

	try{
		std::string testenv=getenv("MAGNETDB"); //trick to get an exception if env is null
		env=testenv;
	}
	catch(std::logic_error& ex){
		throw std::invalid_argument("missing MAGNETDB environment variable");
	}
	try{

		size_t p0=env.find_first_of(":");
		if(p0 == std::string::npos || p0==0)
			throw std::invalid_argument("invalid MAGNETDB format (0)");
		db=env.substr(0,p0);
		env=env.substr(++p0);
        std::cout<< __LINE__ << db << std::endl;

		size_t p1=env.find_first_of(":");
		if(p1 == std::string::npos || p1==0)
			throw std::invalid_argument("invalid MAGNETDB format (1)");
		user=env.substr(0,p1);
		env=env.substr(++p1);
		size_t p2=env.find_first_of(":");
		if(p2 == std::string::npos || p2==0)
			throw std::invalid_argument("invalid MAGNETDB format (2)");

		passw=env.substr(0,p2);
		host=env.substr(++p2);

		std::cout << db << " " << passw << " " << host << std::endl;
	}
	catch(std::logic_error& ex){
		throw std::invalid_argument("uncomplete MAGNETDB environment variable");
	}
	initdb(db.c_str(), user.c_str(),passw.c_str(),host.c_str());
}

Magnetfactory::Magnetfactory(const char* dbname, const char* dbuser, const char* dbpasswd, const char* dbhost)
{
	initdb(dbname, dbuser,dbpasswd,dbhost);
}


Magnetfactory::~Magnetfactory()
{

	mysql_close(&mysql);
	mysql_thread_end();
	mysql_library_end();
}

Quadrupole* Magnetfactory::newQuadrupole(const char* quadname)
{

	Spline* field_spline=0;
	multiPolynomial* field_multipoly=0;
	Interpolator::Interpolator*  field_interpolator=0;

	Spline* invfield_spline=0;
	multiPolynomial* invfield_multipoly=0;
	Interpolator::Interpolator*  invfield_interpolator=0;

	Spline* length_spline=0;
	multiPolynomial* length_multipoly=0;
	Interpolator::Interpolator*  length_interpolator=0;


	// step 1: query "magnet" table and ask for magnet model name and calibration names
	std::stringstream query(std::stringstream::in | std::stringstream::out);
	query << "SELECT model_name,focussing,quad_calibration_name,maglen_calibration_name,quad_polynomial_name,maglen_polynomial_name FROM magnet WHERE name='" << quadname <<"'";

	int myres=mysql_query(&mysql,query.str().c_str()); // @suppress("Method cannot be resolved")
	if(myres != 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	MYSQL_RES* queryresult=mysql_store_result(&mysql);
	if(queryresult == 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	int num_fields=mysql_num_fields(queryresult);

	if(num_fields !=6 ){ //check that we get what we expected
		mysql_free_result(queryresult);
		throw std::range_error("wrong number of fields form SQl query on magnet");
	}
	MYSQL_ROW   row=mysql_fetch_row(queryresult);
	if(mysql_errno(&mysql)){
		mysql_free_result(queryresult);
		throw std::invalid_argument(mysql_error(&mysql));
	}
	if(row==0){
		mysql_free_result(queryresult);
		std::stringstream msg(std::stringstream::out);
		msg << "magnet " << quadname << " not found in database";
		throw std::invalid_argument(msg.str());
	}
	std::string model_name=std::string(row[0]);
	std::string focussing_str=std::string(row[1]);
	bool focussing;
	if(focussing_str== "1") focussing=true;
	else focussing=false;
	std::string quad_calibration_name=std::string(row[2]);
	std::string maglen_calibration_name=std::string(row[3]);
	std::string quad_polynomial_name=std::string(row[4]);
	std::string maglen_polynomial_name=std::string(row[5]);
	mysql_free_result(queryresult);

	//step 2: check that the model class name is correct
	query.str(""); //clean query buffer
	query << "SELECT classname,magnetic_length FROM model WHERE name='" << model_name << "'";
	myres=mysql_query(&mysql,query.str().c_str());
	if(myres != 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	queryresult=mysql_store_result(&mysql);
	if(queryresult == 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	num_fields=mysql_num_fields(queryresult);

	if(num_fields !=2 ){ //check that we get what we expected
		mysql_free_result(queryresult);
		throw std::range_error("wrong number of fields form SQl query on model");
	}
	row=mysql_fetch_row(queryresult);
	std::string classname=std::string(row[0]);
	std::string magnetic_length=std::string(row[1]);
	mysql_free_result(queryresult);

	//check if data is valid
	if(classname != "quadrupole"){
		throw std::range_error("requested magnet is not a quadrupole");
	}

	if( quad_calibration_name == "null" &&  quad_polynomial_name == "null"){
		throw std::invalid_argument("requested quadrupole has no valid field data");
	}

	if(quad_calibration_name != "null"){ //create spline interpolators for the main field
		//step 3a.1: detect and extract the relevant data for field Interpolator::Interpolator
		query.str(""); //clean query buffer
		query << "SELECT curr,quad FROM quad WHERE calibration_name='" << quad_calibration_name << "' ORDER BY curr";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on quad");
		}
		int nrows=mysql_num_rows(queryresult);
		doubleVector curr;
		doubleVector field;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			curr.push_back(strtod(row[0],0));
			field.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for field;
		field_spline = new Spline(curr,field);
		field_interpolator=field_spline;
		mysql_free_result(queryresult);

		//step 3a.2: detect and extract the relevant data for inverse field Interpolator::Interpolator
		query.str(""); //clean query buffer
		query << "SELECT quad,curr FROM quad WHERE calibration_name='" << quad_calibration_name << "' ORDER BY quad";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on quad");
		}
		nrows=mysql_num_rows(queryresult);
		doubleVector icurr;
		doubleVector ifield;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			ifield.push_back(strtod(row[0],0));
			icurr.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for field;
		invfield_spline = new Spline(ifield,icurr);
		invfield_interpolator=invfield_spline;
		mysql_free_result(queryresult);


	}
	else{ //create multi-polynomial interpolators for the main and inverse field
		{
			//step 3b.1: detect and extract the field for multi polynomial interpolators
			query.str(""); //clean query buffer
			query <<"SELECT min_direct,max_direct,degree_direct,d0,d1,d2,d3,d4,d5,d6 FROM polynomial_calibration WHERE polynomial_name='" << quad_polynomial_name <<"' ORDER BY min_direct";
			myres=mysql_query(&mysql,query.str().c_str());
			if(myres != 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			queryresult=mysql_store_result(&mysql);
			if(queryresult == 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			num_fields=mysql_num_fields(queryresult);

			if(num_fields !=10 ){ //check that we get what we expected
				mysql_free_result(queryresult);
				throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
			}
			int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

			InterpolatingPolynoamialVector pvector; //vector of plynomials
			for( int i=0; i< nrows; i++ ){
				InterpolatingPolynomial poly; //temporary polynomial
				row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
				poly.xMin=strtod(row[0],0);
				poly.xMax=strtod(row[1],0);
				int degree=atoi(row[2]);
				for(int k=0; k<=degree;k++){
					poly.coefficient.push_back(strtod(row[k+3],0));
				}
				pvector.push_back(poly); //insert polynomial int vector;
			}
			//crete the multypolynomial
			field_multipoly=new multiPolynomial(pvector);
			field_interpolator=field_multipoly;
			mysql_free_result(queryresult);
		}
		{
			//step 3b.2: detect and extract the field for INVERSE multi polynomial interpolators
			query.str(""); //clean query buffer
			query <<"SELECT min_inverse,max_inverse,degree_inverse,i0,i1,i2,i3,i4,i5,i6 FROM polynomial_calibration WHERE polynomial_name='" << quad_polynomial_name <<"' ORDER BY min_inverse";
			myres=mysql_query(&mysql,query.str().c_str());
			if(myres != 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			queryresult=mysql_store_result(&mysql);
			if(queryresult == 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			num_fields=mysql_num_fields(queryresult);

			if(num_fields !=10 ){ //check that we get what we expected
				mysql_free_result(queryresult);
				throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
			}
			int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

			InterpolatingPolynoamialVector pvector; //vector of plynomials
			for( int i=0; i< nrows; i++ ){
				InterpolatingPolynomial poly; //temporary polynomial
				row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
				poly.xMin=strtod(row[0],0);
				poly.xMax=strtod(row[1],0);
				int degree=atoi(row[2]);
				for(int k=0; k<=degree;k++){
					poly.coefficient.push_back(strtod(row[k+3],0));
				}
				pvector.push_back(poly); //insert polynomial int vector;
			}
			//crete the multypolynomial
			invfield_multipoly=new multiPolynomial(pvector);
			invfield_interpolator=invfield_multipoly;
			mysql_free_result(queryresult);
		}

	}


	if(maglen_calibration_name != "null"){ //create spline interpolators for the magnetic_len
		//step 4a: detect and extract the magnetic length
		query.str(""); //clean query buffer
		query << "SELECT curr,len FROM maglen WHERE calibration_name='" << maglen_calibration_name << "' ORDER BY curr";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on maglen");
		}
		int nrows=mysql_num_rows(queryresult);
		doubleVector curr;
		doubleVector len;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			curr.push_back(strtod(row[0],0));
			len.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for field;
		length_spline = new Spline(curr,len);
		length_interpolator=length_spline;
		mysql_free_result(queryresult);
	}

	//------------------
	if(maglen_polynomial_name != "null"){ //create multipolynomial interpolators for the magnetic_len{
		//step 4b: detect and extract the  multi polynomial interpolators for the magneti length
		query.str(""); //clean query buffer
		query <<"SELECT min_direct,max_direct,degree_direct,d0,d1,d2,d3,d4,d5,d6 FROM polynomial_calibration WHERE polynomial_name='" << maglen_polynomial_name <<"' ORDER BY min_direct";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=10 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
		}
		int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

		InterpolatingPolynoamialVector pvector; //vector of plynomials
		for( int i=0; i< nrows; i++ ){
			InterpolatingPolynomial poly; //temporary polynomial
			row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
			poly.xMin=strtod(row[0],0);
			poly.xMax=strtod(row[1],0);
			int degree=atoi(row[2]);
			for(int k=0; k<=degree;k++){
				poly.coefficient.push_back(strtod(row[k+3],0));
			}
			pvector.push_back(poly); //insert polynomial int vector;
		}
		//crete the multypolynomial
		length_multipoly=new multiPolynomial(pvector);
		length_interpolator=length_multipoly;
		mysql_free_result(queryresult);
	}

	//if neither a spline nor a multipolynomial are provided for magnetic length, use static data from MODEL
	// and create a 0-degree polynomial (that  is constant value).
	if(length_interpolator==0){
		//step 4b magnetic_length
		InterpolatingPolynomial poly; //temporary polynomial
		InterpolatingPolynoamialVector pvector; //vector of plynomials
		double min,max;
		field_interpolator->get_range(min,max);
		poly.xMin=min;
		poly.xMax=max;
		poly.coefficient.push_back(strtod(magnetic_length.c_str(),0));
		pvector.push_back(poly);
		length_multipoly=new multiPolynomial(pvector);
		length_interpolator=length_multipoly;

	}

	//step 5: create the quadrupole and transfer ownership to caller
	Quadrupole* quad= new Quadrupole(focussing,*field_interpolator,*invfield_interpolator,*length_interpolator,quadname);

	quad->setGeometricLength(atoi(magnetic_length.c_str()));


	// 	if (field_spline) delete field_spline;
	// 	if (invfield_spline) delete invfield_spline;
	// 	if (field_multipoly) delete field_multipoly;
	// 	if (invfield_multipoly) delete invfield_multipoly;
	// 	if (length_spline) delete length_spline;
	// 	if (length_multipoly) delete length_multipoly;
	return quad;

}

//-------------------------------
Dipole* Magnetfactory::newDipole(const char* dipolename)
{

	Spline* field_spline=0;
	multiPolynomial* field_multipoly=nullptr;
	Interpolator::Interpolator*  field_interpolator=nullptr;

	Spline* invfield_spline=nullptr;
	multiPolynomial* invfield_multipoly=nullptr;
	Interpolator::Interpolator*  invfield_interpolator=nullptr;

	Spline* length_spline=nullptr;
	multiPolynomial* length_multipoly=nullptr;
	Interpolator::Interpolator*  length_interpolator=nullptr;

	Spline* energy_spline=nullptr;
	multiPolynomial* energy_multipoly=nullptr;
	Interpolator::Interpolator*  energy_interpolator=nullptr;

	Spline* invenergy_spline=nullptr;
	multiPolynomial* invenergy_multipoly=nullptr;
	Interpolator::Interpolator*  invenergy_interpolator=nullptr;

	//Spline* quad_spline=nullptr;
	multiPolynomial* quad_multipoly=nullptr;
	Interpolator::Interpolator*  quad_interpolator=nullptr;

	multiPolynomial* dipoleindex_multipoly=nullptr;
	Interpolator::Interpolator*  dipoleindex_interpolator=nullptr;

	// step 1: query "magnet" table and ask for magnet model name and calibration names
	std::stringstream query(std::stringstream::in | std::stringstream::out);
	query << "SELECT model_name,positive,dipole_calibration_name,maglen_calibration_name,dipole_polynomial_name,maglen_polynomial_name,energy_calibration_name,energy_polynomial_name,quad_calibration_name,quad_polynomial_name,dipoleindex_calibration_name,dipoleindex_polynomial_name FROM magnet WHERE name='" << dipolename <<"'";

	int myres=mysql_query(&mysql,query.str().c_str());
	if(myres != 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	MYSQL_RES* queryresult=mysql_store_result(&mysql);
	if(queryresult == 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	int num_fields=mysql_num_fields(queryresult);

	if(num_fields !=12 ){ //check that we get what we expected
		mysql_free_result(queryresult);
		throw std::range_error("wrong number of fields form SQl query on magnet");
	}
	MYSQL_ROW   row=mysql_fetch_row(queryresult);
	if(mysql_errno(&mysql)){
		mysql_free_result(queryresult);
		throw std::invalid_argument(mysql_error(&mysql));
	}
	if(row==0){
		mysql_free_result(queryresult);
		std::stringstream msg(std::stringstream::out);
		msg << "magnet " << dipolename << " not found in database";
		throw std::invalid_argument(msg.str());
	}
	std::string model_name=std::string(row[0]);
	std::string positive_str=std::string(row[1]);
	//TODO: better analysis of ambiguos cases , test also for null values
	bool positive;
	if(positive_str== "1") positive=true;
	else positive=false;
	std::string dipole_calibration_name=std::string(row[2]);
	std::string maglen_calibration_name=std::string(row[3]);
	std::string dipole_polynomial_name=std::string(row[4]);
	std::string maglen_polynomial_name=std::string(row[5]);
	std::string energy_calibration_name=std::string(row[6]);
	std::string energy_polynomial_name=std::string(row[7]);
	std::string quad_calibration_name=std::string(row[8]);
	std::string quad_polynomial_name=std::string(row[9]);
	std::string dipoleindex_calibration_name=std::string(row[10]);
	std::string dipoleindex_polynomial_name=std::string(row[11]);
	mysql_free_result(queryresult);

	//step 2: check that the model class name is correct
	query.str(""); //clean query buffer
	query << "SELECT classname,magnetic_length,radius,bend_angle FROM model WHERE name='" << model_name << "'";
	myres=mysql_query(&mysql,query.str().c_str());
	if(myres != 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	queryresult=mysql_store_result(&mysql);
	if(queryresult == 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	num_fields=mysql_num_fields(queryresult);

	if(num_fields !=4){ //check that we get what we expected
		mysql_free_result(queryresult);
		throw std::range_error("wrong number of fields form SQl query on model");
	}
	row=mysql_fetch_row(queryresult);
	std::string classname=std::string(row[0]);
	std::string magnetic_length=std::string(row[1]);
	std::string radius=std::string(row[2]);
	std::string bend_angle=std::string(row[3]);
	mysql_free_result(queryresult);

	//check if data is valid
	if(!( classname =="dipole" || classname == "sectordipole") ){
		throw std::range_error("requested magnet is not a dipole");
	}

	if( dipole_calibration_name == "null" &&  dipole_polynomial_name == "null"){
		std::string msg("requested dipole: ");
		msg.append(dipolename);
		msg.append(" has no valid field data");
		throw std::invalid_argument(msg);
	}

	if(dipole_calibration_name != "null"){ //create spline interpolators for the main field
		//step 3a.1: detect and extract the relevant data for field Interpolator::Interpolator
		query.str(""); //clean query buffer
		query << "SELECT curr,dipole FROM dipole WHERE calibration_name='" << dipole_calibration_name << "' ORDER BY curr";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on dipole");
		}
		int nrows=mysql_num_rows(queryresult);
		doubleVector curr;
		doubleVector field;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			curr.push_back(strtod(row[0],0));
			field.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for field;
		field_spline = new Spline(curr,field);
		field_interpolator=field_spline;
		mysql_free_result(queryresult);

		//step 3a.2: detect and extract the relevant data for inverse field Interpolator::Interpolator
		query.str(""); //clean query buffer
		query << "SELECT dipole,curr FROM dipole WHERE calibration_name='" << dipole_calibration_name << "' ORDER BY dipole";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on dipole");
		}
		nrows=mysql_num_rows(queryresult);
		doubleVector icurr;
		doubleVector ifield;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			ifield.push_back(strtod(row[0],0));
			icurr.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for field;
		invfield_spline = new Spline(ifield,icurr);
		invfield_interpolator=invfield_spline;
		mysql_free_result(queryresult);


	}
	else{ //create multi-polynomial interpolators for the main and inverse field
		{
			//step 3b.1: detect and extract the field for multi polynomial interpolators
			query.str(""); //clean query buffer
			query <<"SELECT min_direct,max_direct,degree_direct,d0,d1,d2,d3,d4,d5,d6 FROM polynomial_calibration WHERE polynomial_name='" << dipole_polynomial_name <<"' ORDER BY min_direct";
			myres=mysql_query(&mysql,query.str().c_str());
			if(myres != 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			queryresult=mysql_store_result(&mysql);
			if(queryresult == 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			num_fields=mysql_num_fields(queryresult);

			if(num_fields !=10 ){ //check that we get what we expected
				mysql_free_result(queryresult);
				throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
			}
			int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

			InterpolatingPolynoamialVector pvector; //vector of plynomials
			for( int i=0; i< nrows; i++ ){
				InterpolatingPolynomial poly; //temporary polynomial
				row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
				poly.xMin=strtod(row[0],0);
				poly.xMax=strtod(row[1],0);
				int degree=atoi(row[2]);
				for(int k=0; k<=degree;k++){
					poly.coefficient.push_back(strtod(row[k+3],0));
				}
				pvector.push_back(poly); //insert polynomial int vector;
			}
			//crete the multypolynomial
			field_multipoly=new multiPolynomial(pvector);
			field_interpolator=field_multipoly;
			mysql_free_result(queryresult);
		}
		{
			//step 3b.2: detect and extract the field for INVERSE multi polynomial interpolators
			query.str(""); //clean query buffer
			query <<"SELECT min_inverse,max_inverse,degree_inverse,i0,i1,i2,i3,i4,i5,i6 FROM polynomial_calibration WHERE polynomial_name='" << dipole_polynomial_name <<"' ORDER BY min_inverse";
			myres=mysql_query(&mysql,query.str().c_str());
			if(myres != 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			queryresult=mysql_store_result(&mysql);
			if(queryresult == 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			num_fields=mysql_num_fields(queryresult);

			if(num_fields !=10 ){ //check that we get what we expected
				mysql_free_result(queryresult);
				throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
			}
			int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

			InterpolatingPolynoamialVector pvector; //vector of plynomials
			for( int i=0; i< nrows; i++ ){
				InterpolatingPolynomial poly; //temporary polynomial
				row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
				poly.xMin=strtod(row[0],0);
				poly.xMax=strtod(row[1],0);
				int degree=atoi(row[2]);
				for(int k=0; k<=degree;k++){
					poly.coefficient.push_back(strtod(row[k+3],0));
				}
				pvector.push_back(poly); //insert polynomial int vector;
			}
			//crete the multypolynomial
			invfield_multipoly=new multiPolynomial(pvector);
			invfield_interpolator=invfield_multipoly;
			mysql_free_result(queryresult);
		}

	}


	if(maglen_calibration_name != "null"){ //create spline interpolators for the magnetic_len
		//step 4a: detect and extract the magnetic length
		query.str(""); //clean query buffer
		query << "SELECT curr,len FROM maglen WHERE calibration_name='" << maglen_calibration_name << "' ORDER BY curr";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on maglen");
		}
		int nrows=mysql_num_rows(queryresult);
		doubleVector curr;
		doubleVector len;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			curr.push_back(strtod(row[0],0));
			len.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for field;
		length_spline = new Spline(curr,len);
		length_interpolator=length_spline;
		mysql_free_result(queryresult);
	}

	if(maglen_polynomial_name != "null"){ //create multipolynomial interpolators for the magnetic_len{
		//step 4b: detect and extract the  multi polynomial interpolators for the magneti length
		query.str(""); //clean query buffer
		query <<"SELECT min_direct,max_direct,degree,d0,d1,d2,d3,d4,d5,d6 FROM polynomial_calibration WHERE polynomial_name='" << maglen_polynomial_name <<"' ORDER BY min_direct";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=10 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
		}
		int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

		InterpolatingPolynoamialVector pvector; //vector of plynomials
		for( int i=0; i< nrows; i++ ){
			InterpolatingPolynomial poly; //temporary polynomial
			row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
			poly.xMin=strtod(row[0],0);
			poly.xMax=strtod(row[1],0);
			int degree=atoi(row[2]);
			for(int k=0; k<=degree;k++){
				poly.coefficient.push_back(strtod(row[k+3],0));
			}
			pvector.push_back(poly); //insert polynomial int vector;
		}
		//crete the multypolynomial
		length_multipoly=new multiPolynomial(pvector);
		length_interpolator=length_multipoly;
		mysql_free_result(queryresult);
	}

	//if neither a spline nor a multipolynomial are provided for magnetic length, use static data from MODEL
	// and create a 0-degree polynomial (that  is constant value).
	if(length_interpolator==nullptr){
		//step 4b magnetic_length
		InterpolatingPolynomial poly; //temporary polynomial
		InterpolatingPolynoamialVector pvector; //vector of plynomials
		double min,max;
		field_interpolator->get_range(min,max);
		poly.xMin=min;
		poly.xMax=max;
		poly.coefficient.push_back(strtod(magnetic_length.c_str(),0));
		pvector.push_back(poly);
		length_multipoly=new multiPolynomial(pvector);
		length_interpolator=length_multipoly;

	}

	if(energy_calibration_name != "null"){ //create spline interpolators for the energy
		//step 7a.1: detect and extract the relevant data for energy Interpolator::Interpolator
		query.str(""); //clean query buffer
		query << "SELECT curr,energy FROM energy WHERE calibration_name='" << energy_calibration_name << "' ORDER BY curr";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on energy");
		}
		int nrows=mysql_num_rows(queryresult);
		doubleVector curr;
		doubleVector energy;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			curr.push_back(strtod(row[0],0));
			energy.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for field;
		energy_spline = new Spline(curr,energy);
		energy_interpolator=energy_spline;
		mysql_free_result(queryresult);

		//step 3a.2: detect and extract the relevant data for inverse energy Interpolator::Interpolator
		query.str(""); //clean query buffer
		query << "SELECT energy,curr FROM energy WHERE calibration_name='" << energy_calibration_name << "' ORDER BY energy";

		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on energy");
		}
		nrows=mysql_num_rows(queryresult);
		doubleVector icurr;
		doubleVector ienergy;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			ienergy.push_back(strtod(row[0],0));
			icurr.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for energy;
		invenergy_spline = new Spline(ienergy,icurr);
		invenergy_interpolator=invenergy_spline;
		mysql_free_result(queryresult);


	}
	else{ //create multi-polynomial interpolators for the main and inverse energy
		{
			//step 7b.1: detect and extract the field for multi polynomial energy interpolators
			query.str(""); //clean query buffer
			query <<"SELECT min_direct,max_direct,degree_direct,d0,d1,d2,d3,d4,d5,d6 FROM polynomial_calibration WHERE polynomial_name='" << energy_polynomial_name <<"' ORDER BY min_direct";
			myres=mysql_query(&mysql,query.str().c_str());
			if(myres != 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			queryresult=mysql_store_result(&mysql);
			if(queryresult == 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			num_fields=mysql_num_fields(queryresult);

			if(num_fields !=10 ){ //check that we get what we expected
				mysql_free_result(queryresult);
				throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
			}
			int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

			InterpolatingPolynoamialVector pvector; //vector of plynomials
			for( int i=0; i< nrows; i++ ){
				InterpolatingPolynomial poly; //temporary polynomial
				row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
				poly.xMin=strtod(row[0],0);
				poly.xMax=strtod(row[1],0);
				int degree=atoi(row[2]);
				for(int k=0; k<=degree;k++){
					poly.coefficient.push_back(strtod(row[k+3],0));
				}
				pvector.push_back(poly); //insert polynomial int vector;
			}
			//crete the multypolynomial
			energy_multipoly=new multiPolynomial(pvector);
			energy_interpolator=energy_multipoly;
			mysql_free_result(queryresult);
		}
		{
			//step 7b.2: detect and extract the energy for INVERSE multi polynomial interpolators
			query.str(""); //clean query buffer
			query <<"SELECT min_inverse,max_inverse,degree_inverse,i0,i1,i2,i3,i4,i5,i6 FROM polynomial_calibration WHERE polynomial_name='" << energy_polynomial_name <<"' ORDER BY min_inverse";
			myres=mysql_query(&mysql,query.str().c_str());
			if(myres != 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			queryresult=mysql_store_result(&mysql);
			if(queryresult == 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			num_fields=mysql_num_fields(queryresult);

			if(num_fields !=10 ){ //check that we get what we expected
				mysql_free_result(queryresult);
				throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
			}
			int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

			InterpolatingPolynoamialVector pvector; //vector of plynomials
			for( int i=0; i< nrows; i++ ){
				InterpolatingPolynomial poly; //temporary polynomial
				row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
				poly.xMin=strtod(row[0],0);
				poly.xMax=strtod(row[1],0);
				int degree=atoi(row[2]);
				for(int k=0; k<=degree;k++){
					poly.coefficient.push_back(strtod(row[k+3],0));
				}
				pvector.push_back(poly); //insert polynomial int vector;
			}
			//crete the multypolynomial
			invenergy_multipoly=new multiPolynomial(pvector);
			invenergy_interpolator=invenergy_multipoly;
			mysql_free_result(queryresult);
		}

	}

	//==============================================================
	if(quad_calibration_name != "null"){ //create spline interpolators for the gradient field
		//step 8a: detect and extract the relevant data for gradient Interpolator::Interpolator
		query.str(""); //clean query buffer
		query << "SELECT curr,quad FROM quad WHERE calibration_name='" << dipole_calibration_name << "' ORDER BY curr";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on quad");
		}
		int nrows=mysql_num_rows(queryresult);
		doubleVector curr;
		doubleVector quad;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			curr.push_back(strtod(row[0],0));
			quad.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for quad;
		Spline* quad_spline=nullptr;
		quad_spline = new Spline(curr,quad);
		quad_interpolator=quad_spline;
		mysql_free_result(queryresult);
	}
	else if(quad_polynomial_name != "null"){ //create multi-polynomial interpolators for the gradient field
		{
			//step 8b: detect and extract the field for multi polynomial interpolators
			query.str(""); //clean query buffer
			query <<"SELECT min_direct,max_direct,degree_direct,d0,d1,d2,d3,d4,d5,d6 FROM polynomial_calibration WHERE polynomial_name='" << quad_polynomial_name <<"' ORDER BY min_direct";
			myres=mysql_query(&mysql,query.str().c_str());
			if(myres != 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			queryresult=mysql_store_result(&mysql);
			if(queryresult == 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			num_fields=mysql_num_fields(queryresult);

			if(num_fields !=10 ){ //check that we get what we expected
				mysql_free_result(queryresult);
				throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
			}
			int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

			Interpolator::InterpolatingPolynoamialVector pvector; //vector of plynomials
			for( int i=0; i< nrows; i++ ){
				Interpolator::InterpolatingPolynomial poly; //temporary polynomial
				row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
				poly.xMin=strtod(row[0],0);
				poly.xMax=strtod(row[1],0);
				int degree=atoi(row[2]);
				for(int k=0; k<=degree;k++){
					poly.coefficient.push_back(strtod(row[k+3],0));
				}
				pvector.push_back(poly); //insert polynomial int vector;
			}
			//crete the multypolynomial
			quad_multipoly=new Interpolator::multiPolynomial(pvector);
			quad_interpolator=quad_multipoly;
			mysql_free_result(queryresult);
		}

	}
	//==============================================================
	if(dipoleindex_calibration_name != "null"){ //create spline interpolators for the index
		//step 9a: detect and extract the relevant data for index Interpolator::Interpolator
		query.str(""); //clean query buffer
		query << "SELECT curr,dipoleindex FROM dipoleindex WHERE calibration_name='" << dipoleindex_calibration_name << "' ORDER BY curr";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on dipoleindex");
		}
		int nrows=mysql_num_rows(queryresult);
		doubleVector curr;
		doubleVector dipoleindex;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			curr.push_back(strtod(row[0],0));
			dipoleindex.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for index;
		Spline* dipoleindex_spline=nullptr;
		dipoleindex_spline = new Spline(curr,dipoleindex);
		dipoleindex_interpolator = dipoleindex_spline; //TODO: cheCk!!!
		mysql_free_result(queryresult);
	}
	else if(dipoleindex_polynomial_name != "null"){ //create multi-polynomial interpolators for the index
		{
			//step 9b: detect and extract  multi polynomial interpolators for index
			query.str(""); //clean query buffer
			query <<"SELECT min_direct,max_direct,degree_direct,d0,d1,d2,d3,d4,d5,d6 FROM polynomial_calibration WHERE polynomial_name='" << dipoleindex_polynomial_name <<"' ORDER BY min_direct";
			myres=mysql_query(&mysql,query.str().c_str());
			if(myres != 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			queryresult=mysql_store_result(&mysql);
			if(queryresult == 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			num_fields=mysql_num_fields(queryresult);

			if(num_fields !=10 ){ //check that we get what we expected
				mysql_free_result(queryresult);
				throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
			}
			int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

			InterpolatingPolynoamialVector pvector; //vector of plynomials
			for( int i=0; i< nrows; i++ ){
				InterpolatingPolynomial poly; //temporary polynomial
				row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
				poly.xMin=strtod(row[0],0);
				poly.xMax=strtod(row[1],0);
				int degree=atoi(row[2]);
				for(int k=0; k<=degree;k++){
					poly.coefficient.push_back(strtod(row[k+3],0));
				}
				pvector.push_back(poly); //insert polynomial int vector;
			}
			//crete the multypolynomial
			dipoleindex_multipoly=new multiPolynomial(pvector);
			dipoleindex_interpolator=dipoleindex_multipoly;
			mysql_free_result(queryresult);
		}

	}
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	//step 10: create the dipole
	Dipole* dipole= new Dipole(positive,*field_interpolator,*invfield_interpolator,*length_interpolator,*energy_interpolator,*invenergy_interpolator,dipolename);

	dipole->setGeometricLength(strtod(magnetic_length.c_str(),0));
	dipole->setRadius(strtod(radius.c_str(),0));
	dipole->setAngle(strtod(bend_angle.c_str(),0));

	//step 11: chekc if the magnet has gradient interopolators and set them now
	if(quad_interpolator) dipole->setGradientInterpolator(*quad_interpolator);
	if(dipoleindex_interpolator) dipole->setIndexInterpolator(*dipoleindex_interpolator);



	// 	if (field_spline) delete field_spline;
	// 	if (invfield_spline) delete invfield_spline;
	// 	if (field_multipoly) delete field_multipoly;
	// 	if (invfield_multipoly) delete invfield_multipoly;
	// 	if (length_spline) delete length_spline;
	// 	if (length_multipoly) delete length_multipoly;
	//step 12: reutnr new Dipole and transfer ownership
	return dipole;

}

//==========================================
Sextupole* Magnetfactory::newSextupole(const char* sextname)
{

	Spline* field_spline=0;
	multiPolynomial* field_multipoly=0;
	Interpolator::Interpolator*  field_interpolator=0;

	Spline* invfield_spline=0;
	multiPolynomial* invfield_multipoly=0;
	Interpolator::Interpolator*  invfield_interpolator=0;

	Spline* length_spline=0;
	multiPolynomial* length_multipoly=0;
	Interpolator::Interpolator*  length_interpolator=0;


	// step 1: query "magnet" table and ask for magnet model name and calibration names
	std::stringstream query(std::stringstream::in | std::stringstream::out);
	query << "SELECT model_name,focussing,sext_calibration_name,maglen_calibration_name,sext_polynomial_name,maglen_polynomial_name FROM magnet WHERE name='" << sextname <<"'";

	int myres=mysql_query(&mysql,query.str().c_str());
	if(myres != 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	MYSQL_RES* queryresult=mysql_store_result(&mysql);
	if(queryresult == 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	int num_fields=mysql_num_fields(queryresult);

	if(num_fields !=6 ){ //check that we get what we expected
		mysql_free_result(queryresult);
		throw std::range_error("wrong number of fields form SQl query on magnet");
	}
	MYSQL_ROW   row=mysql_fetch_row(queryresult);
	if(mysql_errno(&mysql)){
		mysql_free_result(queryresult);
		throw std::invalid_argument(mysql_error(&mysql));
	}
	if(row==0){
		mysql_free_result(queryresult);
		std::stringstream msg(std::stringstream::out);
		msg << "magnet " << sextname << " not found in database";
		throw std::invalid_argument(msg.str());
	}
	std::string model_name=std::string(row[0]);
	std::string focussing_str=std::string(row[1]);
	bool focussing;
	if(focussing_str== "1") focussing=true;
	else focussing=false;
	std::string sext_calibration_name=std::string(row[2]);
	std::string maglen_calibration_name=std::string(row[3]);
	std::string sext_polynomial_name=std::string(row[4]);
	std::string maglen_polynomial_name=std::string(row[5]);
	mysql_free_result(queryresult);

	//step 2: check that the model class name is correct
	query.str(""); //clean query buffer
	query << "SELECT classname,magnetic_length FROM model WHERE name='" << model_name << "'";
	myres=mysql_query(&mysql,query.str().c_str());
	if(myres != 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	queryresult=mysql_store_result(&mysql);
	if(queryresult == 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	num_fields=mysql_num_fields(queryresult);

	if(num_fields !=2 ){ //check that we get what we expected
		mysql_free_result(queryresult);
		throw std::range_error("wrong number of fields form SQl query on model");
	}
	row=mysql_fetch_row(queryresult);
	std::string classname=std::string(row[0]);
	std::string magnetic_length=std::string(row[1]);
	mysql_free_result(queryresult);

	//check if data is valid
	if(classname != "sextupole"){
		throw std::range_error("requested magnet is not a sextupole");
	}

	if( sext_calibration_name == "null" &&  sext_polynomial_name == "null"){
		throw std::invalid_argument("requested sextupole has no valid field data");
	}

	if(sext_calibration_name != "null"){ //create spline interpolators for the main field
		//step 3a.1: detect and extract the relevant data for field Interpolator::Interpolator
		query.str(""); //clean query buffer
		query << "SELECT curr,sext FROM sext WHERE calibration_name='" << sext_calibration_name << "' ORDER BY curr";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on sext");
		}
		int nrows=mysql_num_rows(queryresult);
		doubleVector curr;
		doubleVector field;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			curr.push_back(strtod(row[0],0));
			field.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for field;
		field_spline = new Spline(curr,field);
		field_interpolator=field_spline;
		mysql_free_result(queryresult);

		//step 3a.2: detect and extract the relevant data for inverse field Interpolator::Interpolator
		query.str(""); //clean query buffer
		query << "SELECT sext,curr FROM sext WHERE calibration_name='" << sext_calibration_name << "' ORDER BY sext";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on sext");
		}
		nrows=mysql_num_rows(queryresult);
		doubleVector icurr;
		doubleVector ifield;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			ifield.push_back(strtod(row[0],0));
			icurr.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for field;
		invfield_spline = new Spline(ifield,icurr);
		invfield_interpolator=invfield_spline;
		mysql_free_result(queryresult);


	}
	else{ //create multi-polynomial interpolators for the main and inverse field
		{
			//step 3b.1: detect and extract the field for multi polynomial interpolators
			query.str(""); //clean query buffer
			query <<"SELECT min_direct,max_direct,degree_direct,d0,d1,d2,d3,d4,d5,d6 FROM polynomial_calibration WHERE polynomial_name='" << sext_polynomial_name <<"' ORDER BY min_direct";
			myres=mysql_query(&mysql,query.str().c_str());
			if(myres != 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			queryresult=mysql_store_result(&mysql);
			if(queryresult == 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			num_fields=mysql_num_fields(queryresult);

			if(num_fields !=10 ){ //check that we get what we expected
				mysql_free_result(queryresult);
				throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
			}
			int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

			InterpolatingPolynoamialVector pvector; //vector of plynomials
			for( int i=0; i< nrows; i++ ){
				InterpolatingPolynomial poly; //temporary polynomial
				row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
				poly.xMin=strtod(row[0],0);
				poly.xMax=strtod(row[1],0);
				int degree=atoi(row[2]);
				for(int k=0; k<=degree;k++){
					poly.coefficient.push_back(strtod(row[k+3],0));
				}
				pvector.push_back(poly); //insert polynomial int vector;
			}
			//crete the multypolynomial
			field_multipoly=new multiPolynomial(pvector);
			field_interpolator=field_multipoly;
			mysql_free_result(queryresult);
		}
		{
			//step 3b.2: detect and extract the field for INVERSE multi polynomial interpolators
			query.str(""); //clean query buffer
			query <<"SELECT min_inverse,max_inverse,degree_inverse,i0,i1,i2,i3,i4,i5,i6 FROM polynomial_calibration WHERE polynomial_name='" << sext_polynomial_name <<"' ORDER BY min_inverse";
			myres=mysql_query(&mysql,query.str().c_str());
			if(myres != 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			queryresult=mysql_store_result(&mysql);
			if(queryresult == 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			num_fields=mysql_num_fields(queryresult);

			if(num_fields !=10 ){ //check that we get what we expected
				mysql_free_result(queryresult);
				throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
			}
			int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

			InterpolatingPolynoamialVector pvector; //vector of plynomials
			for( int i=0; i< nrows; i++ ){
				InterpolatingPolynomial poly; //temporary polynomial
				row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
				poly.xMin=strtod(row[0],0);
				poly.xMax=strtod(row[1],0);
				int degree=atoi(row[2]);
				for(int k=0; k<=degree;k++){
					poly.coefficient.push_back(strtod(row[k+3],0));
				}
				pvector.push_back(poly); //insert polynomial int vector;
			}
			//crete the multypolynomial
			invfield_multipoly=new multiPolynomial(pvector);
			invfield_interpolator=invfield_multipoly;
			mysql_free_result(queryresult);
		}

	}


	if(maglen_calibration_name != "null"){ //create spline interpolators for the magnetic_len
		//step 4a: detect and extract the magnetic length
		query.str(""); //clean query buffer
		query << "SELECT curr,len FROM maglen WHERE calibration_name='" << maglen_calibration_name << "' ORDER BY curr";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on maglen");
		}
		int nrows=mysql_num_rows(queryresult);
		doubleVector curr;
		doubleVector len;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			curr.push_back(strtod(row[0],0));
			len.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for field;
		length_spline = new Spline(curr,len);
		length_interpolator=length_spline;
		mysql_free_result(queryresult);
	}

	//------------------
	if(maglen_polynomial_name != "null"){ //create multipolynomial interpolators for the magnetic_len{
		//step 4b: detect and extract the  multi polynomial interpolators for the magneti length
		query.str(""); //clean query buffer
		query <<"SELECT min_direct,max_direct,degree,d0,d1,d2,d3,d4,d5,d6 FROM polynomial_calibration WHERE polynomial_name='" << maglen_polynomial_name <<"' ORDER BY min_direct";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=10 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
		}
		int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

		InterpolatingPolynoamialVector pvector; //vector of plynomials
		for( int i=0; i< nrows; i++ ){
			InterpolatingPolynomial poly; //temporary polynomial
			row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
			poly.xMin=strtod(row[0],0);
			poly.xMax=strtod(row[1],0);
			int degree=atoi(row[2]);
			for(int k=0; k<=degree;k++){
				poly.coefficient.push_back(strtod(row[k+3],0));
			}
			pvector.push_back(poly); //insert polynomial int vector;
		}
		//crete the multypolynomial
		length_multipoly=new multiPolynomial(pvector);
		length_interpolator=length_multipoly;
		mysql_free_result(queryresult);
	}

	//if neither a spline nor a multipolynomial are provided for magnetic length, use static data from MODEL
	// and create a 0-degree polynomial (that  is constant value).
	if(length_interpolator==0){
		//step 4b magnetic_length
		InterpolatingPolynomial poly; //temporary polynomial
		InterpolatingPolynoamialVector pvector; //vector of plynomials
		double min,max;
		field_interpolator->get_range(min,max);
		poly.xMin=min;
		poly.xMax=max;
		poly.coefficient.push_back(strtod(magnetic_length.c_str(),0));
		pvector.push_back(poly);
		length_multipoly=new multiPolynomial(pvector);
		length_interpolator=length_multipoly;

	}

	//step 5: create the sextupole and transfer ownership to caller
	Sextupole* sext= new Sextupole(focussing,*field_interpolator,*invfield_interpolator,*length_interpolator,sextname);

	sext->setGeometricLength(atoi(magnetic_length.c_str()));


	// 	if (field_spline) delete field_spline;
	// 	if (invfield_spline) delete invfield_spline;
	// 	if (field_multipoly) delete field_multipoly;
	// 	if (invfield_multipoly) delete invfield_multipoly;
	// 	if (length_spline) delete length_spline;
	// 	if (length_multipoly) delete length_multipoly;
	return sext;

}
//==========================================
Kicker* Magnetfactory::newKicker(const char* kickname)
{

	Spline* field_spline=0;
	multiPolynomial* field_multipoly=0;
	Interpolator::Interpolator*  field_interpolator=0;

	Spline* invfield_spline=0;
	multiPolynomial* invfield_multipoly=0;
	Interpolator::Interpolator*  invfield_interpolator=0;

	Spline* length_spline=0;
	multiPolynomial* length_multipoly=0;
	Interpolator::Interpolator*  length_interpolator=0;


	// step 1: query "magnet" table and ask for magnet model name and calibration names
	std::stringstream query(std::stringstream::in | std::stringstream::out);
	query << "SELECT model_name,positive,kick_calibration_name,maglen_calibration_name,kick_polynomial_name,maglen_polynomial_name FROM magnet WHERE name='" << kickname <<"'";

	int myres=mysql_query(&mysql,query.str().c_str());
	if(myres != 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	MYSQL_RES* queryresult=mysql_store_result(&mysql);
	if(queryresult == 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	int num_fields=mysql_num_fields(queryresult);

	if(num_fields !=6 ){ //check that we get what we expected
		mysql_free_result(queryresult);
		throw std::range_error("wrong number of fields form SQl query on magnet");
	}
	MYSQL_ROW   row=mysql_fetch_row(queryresult);
	if(mysql_errno(&mysql)){
		mysql_free_result(queryresult);
		throw std::invalid_argument(mysql_error(&mysql));
	}
	if(row==0){
		mysql_free_result(queryresult);
		std::stringstream msg(std::stringstream::out);
		msg << "magnet " << kickname << " not found in database";
		throw std::invalid_argument(msg.str());
	}
	std::string model_name=std::string(row[0]);
	std::string positive_str=std::string(row[1]);
	bool positive;
	if(positive_str== "1") positive=true;
	else positive=false;
	std::string kick_calibration_name=std::string(row[2]);
	std::string maglen_calibration_name=std::string(row[3]);
	std::string kick_polynomial_name=std::string(row[4]);
	std::string maglen_polynomial_name=std::string(row[5]);
	mysql_free_result(queryresult);

	//step 2: check that the model class name is correct
	query.str(""); //clean query buffer
	query << "SELECT classname,magnetic_length FROM model WHERE name='" << model_name << "'";
	myres=mysql_query(&mysql,query.str().c_str());
	if(myres != 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	queryresult=mysql_store_result(&mysql);
	if(queryresult == 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	num_fields=mysql_num_fields(queryresult);

	if(num_fields !=2 ){ //check that we get what we expected
		mysql_free_result(queryresult);
		throw std::range_error("wrong number of fields form SQl query on model");
	}
	row=mysql_fetch_row(queryresult);
	std::string classname=std::string(row[0]);
	std::string magnetic_length=std::string(row[1]);
	mysql_free_result(queryresult);

	//check if data is valid
	if(!(classname == "kicker" || classname == "corrh" || classname == "corrv")){
		throw std::range_error("requested magnet is not a kicker/corrector");
	}

	if( kick_calibration_name == "null" &&  kick_polynomial_name == "null"){
		std::string msg=std::string(kickname);
		msg+=": requested kicker has no valid field data";
		//throw std::invalid_argument("requested kicker has no valid field data");
		throw std::invalid_argument(msg);
	}

	if(kick_calibration_name != "null"){ //create spline interpolators for the main field
		//step 3a.1: detect and extract the relevant data for field Interpolator::Interpolator
		query.str(""); //clean query buffer
		query << "SELECT curr,kick FROM kick WHERE calibration_name='" << kick_calibration_name << "' ORDER BY curr";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on kick");
		}
		int nrows=mysql_num_rows(queryresult);
		doubleVector curr;
		doubleVector field;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			curr.push_back(strtod(row[0],0));
			field.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for field;
		field_spline = new Spline(curr,field);
		field_interpolator=field_spline;
		mysql_free_result(queryresult);

		//step 3a.2: detect and extract the relevant data for inverse field Interpolator::Interpolator
		query.str(""); //clean query buffer
		query << "SELECT kick,curr FROM kick WHERE calibration_name='" << kick_calibration_name << "' ORDER BY kick";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on kick");
		}
		nrows=mysql_num_rows(queryresult);
		doubleVector icurr;
		doubleVector ifield;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			ifield.push_back(strtod(row[0],0));
			icurr.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for field;
		invfield_spline = new Spline(ifield,icurr);
		invfield_interpolator=invfield_spline;
		mysql_free_result(queryresult);


	}
	else{ //create multi-polynomial interpolators for the main and inverse field
		{
			//step 3b.1: detect and extract the field for multi polynomial interpolators
			query.str(""); //clean query buffer
			query <<"SELECT min_direct,max_direct,degree_direct,d0,d1,d2,d3,d4,d5,d6 FROM polynomial_calibration WHERE polynomial_name='" << kick_polynomial_name <<"' ORDER BY min_direct";
			myres=mysql_query(&mysql,query.str().c_str());
			if(myres != 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			queryresult=mysql_store_result(&mysql);
			if(queryresult == 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			num_fields=mysql_num_fields(queryresult);

			if(num_fields !=10 ){ //check that we get what we expected
				mysql_free_result(queryresult);
				throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
			}
			int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

			InterpolatingPolynoamialVector pvector; //vector of plynomials
			for( int i=0; i< nrows; i++ ){
				InterpolatingPolynomial poly; //temporary polynomial
				row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
				poly.xMin=strtod(row[0],0);
				poly.xMax=strtod(row[1],0);
				int degree=atoi(row[2]);
				for(int k=0; k<=degree;k++){
					poly.coefficient.push_back(strtod(row[k+3],0));
				}
				pvector.push_back(poly); //insert polynomial int vector;
			}
			//crete the multypolynomial
			field_multipoly=new multiPolynomial(pvector);
			field_interpolator=field_multipoly;
			mysql_free_result(queryresult);
		}
		{
			//step 3b.2: detect and extract the field for INVERSE multi polynomial interpolators
			query.str(""); //clean query buffer
			query <<"SELECT min_inverse,max_inverse,degree_inverse,i0,i1,i2,i3,i4,i5,i6 FROM polynomial_calibration WHERE polynomial_name='" << kick_polynomial_name <<"' ORDER BY min_inverse";
			myres=mysql_query(&mysql,query.str().c_str());
			if(myres != 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			queryresult=mysql_store_result(&mysql);
			if(queryresult == 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			num_fields=mysql_num_fields(queryresult);

			if(num_fields !=10 ){ //check that we get what we expected
				mysql_free_result(queryresult);
				throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
			}
			int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

			InterpolatingPolynoamialVector pvector; //vector of plynomials
			for( int i=0; i< nrows; i++ ){
				InterpolatingPolynomial poly; //temporary polynomial
				row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
				poly.xMin=strtod(row[0],0);
				poly.xMax=strtod(row[1],0);
				int degree=atoi(row[2]);
				for(int k=0; k<=degree;k++){
					poly.coefficient.push_back(strtod(row[k+3],0));
				}
				pvector.push_back(poly); //insert polynomial int vector;
			}
			//crete the multypolynomial
			invfield_multipoly=new multiPolynomial(pvector);
			invfield_interpolator=invfield_multipoly;
			mysql_free_result(queryresult);
		}

	}


	if(maglen_calibration_name != "null"){ //create spline interpolators for the magnetic_len
		//step 4a: detect and extract the magnetic length
		query.str(""); //clean query buffer
		query << "SELECT curr,len FROM maglen WHERE calibration_name='" << maglen_calibration_name << "' ORDER BY curr";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on maglen");
		}
		int nrows=mysql_num_rows(queryresult);
		doubleVector curr;
		doubleVector len;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			curr.push_back(strtod(row[0],0));
			len.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for field;
		length_spline = new Spline(curr,len);
		length_interpolator=length_spline;
		mysql_free_result(queryresult);
	}

	//------------------
	if(maglen_polynomial_name != "null"){ //create multipolynomial interpolators for the magnetic_len{
		//step 4b: detect and extract the  multi polynomial interpolators for the magneti length
		query.str(""); //clean query buffer
		query <<"SELECT min_direct,max_direct,degree,d0,d1,d2,d3,d4,d5,d6 FROM polynomial_calibration WHERE polynomial_name='" << maglen_polynomial_name <<"' ORDER BY min_direct";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=10 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
		}
		int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

		InterpolatingPolynoamialVector pvector; //vector of plynomials
		for( int i=0; i< nrows; i++ ){
			InterpolatingPolynomial poly; //temporary polynomial
			row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
			poly.xMin=strtod(row[0],0);
			poly.xMax=strtod(row[1],0);
			int degree=atoi(row[2]);
			for(int k=0; k<=degree;k++){
				poly.coefficient.push_back(strtod(row[k+3],0));
			}
			pvector.push_back(poly); //insert polynomial int vector;
		}
		//crete the multypolynomial
		length_multipoly=new multiPolynomial(pvector);
		length_interpolator=length_multipoly;
		mysql_free_result(queryresult);
	}

	//if neither a spline nor a multipolynomial are provided for magnetic length, use static data from MODEL
	// and create a 0-degree polynomial (that  is constant value).
	if(length_interpolator==0){
		//step 4b magnetic_length
		InterpolatingPolynomial poly; //temporary polynomial
		InterpolatingPolynoamialVector pvector; //vector of plynomials
		double min,max;
		field_interpolator->get_range(min,max);
		poly.xMin=min;
		poly.xMax=max;
		poly.coefficient.push_back(strtod(magnetic_length.c_str(),0));
		pvector.push_back(poly);
		length_multipoly=new multiPolynomial(pvector);
		length_interpolator=length_multipoly;

	}

	//step 5: create the kicker and transfer ownership to caller
	Kicker* kick= new Kicker(positive,*field_interpolator,*invfield_interpolator,*length_interpolator,kickname);

	kick->setGeometricLength(atoi(magnetic_length.c_str()));


	// 	if (field_spline) delete field_spline;
	// 	if (invfield_spline) delete invfield_spline;
	// 	if (field_multipoly) delete field_multipoly;
	// 	if (invfield_multipoly) delete invfield_multipoly;
	// 	if (length_spline) delete length_spline;
	// 	if (length_multipoly) delete length_multipoly;
	return kick;

}

//++++++++++++++++++++++++++++++++++++++++++
//==========================================
Solenoid* Magnetfactory::newSolenoid(const char* solenname)
{

	Spline* field_spline=0;
	multiPolynomial* field_multipoly=0;
	Interpolator::Interpolator*  field_interpolator=0;

	Spline* invfield_spline=0;
	multiPolynomial* invfield_multipoly=0;
	Interpolator::Interpolator*  invfield_interpolator=0;

	Spline* length_spline=0;
	multiPolynomial* length_multipoly=0;
	Interpolator::Interpolator*  length_interpolator=0;


	// step 1: query "magnet" table and ask for magnet model name and calibration names
	std::stringstream query(std::stringstream::in | std::stringstream::out);
	query << "SELECT model_name,focussing,solen_calibration_name,maglen_calibration_name,solen_polynomial_name,maglen_polynomial_name FROM magnet WHERE name='" << solenname <<"'";

	int myres=mysql_query(&mysql,query.str().c_str());
	if(myres != 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	MYSQL_RES* queryresult=mysql_store_result(&mysql);
	if(queryresult == 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	int num_fields=mysql_num_fields(queryresult);

	if(num_fields !=6 ){ //check that we get what we expected
		mysql_free_result(queryresult);
		throw std::range_error("wrong number of fields form SQl query on magnet");
	}
	MYSQL_ROW   row=mysql_fetch_row(queryresult);
	if(mysql_errno(&mysql)){
		mysql_free_result(queryresult);
		throw std::invalid_argument(mysql_error(&mysql));
	}
	if(row==0){
		mysql_free_result(queryresult);
		std::stringstream msg(std::stringstream::out);
		msg << "magnet " << solenname << " not found in database";
		throw std::invalid_argument(msg.str());
	}
	std::string model_name=std::string(row[0]);
	std::string focussing_str=std::string(row[1]);
	bool focussing;
	if(focussing_str== "1") focussing=true;
	else focussing=false;
	std::string solen_calibration_name=std::string(row[2]);
	std::string maglen_calibration_name=std::string(row[3]);
	std::string solen_polynomial_name=std::string(row[4]);
	std::string maglen_polynomial_name=std::string(row[5]);
	mysql_free_result(queryresult);

	//step 2: check that the model class name is correct
	query.str(""); //clean query buffer
	query << "SELECT classname,magnetic_length FROM model WHERE name='" << model_name << "'";
	myres=mysql_query(&mysql,query.str().c_str());
	if(myres != 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	queryresult=mysql_store_result(&mysql);
	if(queryresult == 0){
		throw std::invalid_argument(mysql_error(&mysql));
	}
	num_fields=mysql_num_fields(queryresult);

	if(num_fields !=2 ){ //check that we get what we expected
		mysql_free_result(queryresult);
		throw std::range_error("wrong number of fields form SQl query on model");
	}
	row=mysql_fetch_row(queryresult);
	std::string classname=std::string(row[0]);
	std::string magnetic_length=std::string(row[1]);
	mysql_free_result(queryresult);

	//check if data is valid
	if(classname != "solenoid"){
		throw std::range_error("requested magnet is not a solenoid");
	}

	if( solen_calibration_name == "null" &&  solen_polynomial_name == "null"){
		throw std::invalid_argument("requested solenoid has no valid field data");
	}

	if(solen_calibration_name != "null"){ //create spline interpolators for the main field
		//step 3a.1: detect and extract the relevant data for field Interpolator::Interpolator
		query.str(""); //clean query buffer
		query << "SELECT curr,solen FROM solen WHERE calibration_name='" << solen_calibration_name << "' ORDER BY curr";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on solen");
		}
		int nrows=mysql_num_rows(queryresult);
		doubleVector curr;
		doubleVector field;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			curr.push_back(strtod(row[0],0));
			field.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for field;
		field_spline = new Spline(curr,field);
		field_interpolator=field_spline;
		mysql_free_result(queryresult);

		//step 3a.2: detect and extract the relevant data for inverse field Interpolator::Interpolator
		query.str(""); //clean query buffer
		query << "SELECT solen,curr FROM solen WHERE calibration_name='" << solen_calibration_name << "' ORDER BY solen";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on solen");
		}
		nrows=mysql_num_rows(queryresult);
		doubleVector icurr;
		doubleVector ifield;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			ifield.push_back(strtod(row[0],0));
			icurr.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for field;
		invfield_spline = new Spline(ifield,icurr);
		invfield_interpolator=invfield_spline;
		mysql_free_result(queryresult);


	}
	else{ //create multi-polynomial interpolators for the main and inverse field
		{
			//step 3b.1: detect and extract the field for multi polynomial interpolators
			query.str(""); //clean query buffer
			query <<"SELECT min_direct,max_direct,degree_direct,d0,d1,d2,d3,d4,d5,d6 FROM polynomial_calibration WHERE polynomial_name='" << solen_polynomial_name <<"' ORDER BY min_direct";
			myres=mysql_query(&mysql,query.str().c_str());
			if(myres != 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			queryresult=mysql_store_result(&mysql);
			if(queryresult == 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			num_fields=mysql_num_fields(queryresult);

			if(num_fields !=10 ){ //check that we get what we expected
				mysql_free_result(queryresult);
				throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
			}
			int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

			InterpolatingPolynoamialVector pvector; //vector of plynomials
			for( int i=0; i< nrows; i++ ){
				InterpolatingPolynomial poly; //temporary polynomial
				row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
				poly.xMin=strtod(row[0],0);
				poly.xMax=strtod(row[1],0);
				int degree=atoi(row[2]);
				for(int k=0; k<=degree;k++){
					poly.coefficient.push_back(strtod(row[k+3],0));
				}
				pvector.push_back(poly); //insert polynomial int vector;
			}
			//crete the multypolynomial
			field_multipoly=new multiPolynomial(pvector);
			field_interpolator=field_multipoly;
			mysql_free_result(queryresult);
		}
		{
			//step 3b.2: detect and extract the field for INVERSE multi polynomial interpolators
			query.str(""); //clean query buffer
			query <<"SELECT min_inverse,max_inverse,degree_inverse,i0,i1,i2,i3,i4,i5,i6 FROM polynomial_calibration WHERE polynomial_name='" << solen_polynomial_name <<"' ORDER BY min_inverse";
			myres=mysql_query(&mysql,query.str().c_str());
			if(myres != 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			queryresult=mysql_store_result(&mysql);
			if(queryresult == 0){
				throw std::invalid_argument(mysql_error(&mysql));
			}
			num_fields=mysql_num_fields(queryresult);

			if(num_fields !=10 ){ //check that we get what we expected
				mysql_free_result(queryresult);
				throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
			}
			int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

			InterpolatingPolynoamialVector pvector; //vector of plynomials
			for( int i=0; i< nrows; i++ ){
				InterpolatingPolynomial poly; //temporary polynomial
				row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
				poly.xMin=strtod(row[0],0);
				poly.xMax=strtod(row[1],0);
				int degree=atoi(row[2]);
				for(int k=0; k<=degree;k++){
					poly.coefficient.push_back(strtod(row[k+3],0));
				}
				pvector.push_back(poly); //insert polynomial int vector;
			}
			//crete the multypolynomial
			invfield_multipoly=new multiPolynomial(pvector);
			invfield_interpolator=invfield_multipoly;
			mysql_free_result(queryresult);
		}

	}


	if(maglen_calibration_name != "null"){ //create spline interpolators for the magnetic_len
		//step 4a: detect and extract the magnetic length
		query.str(""); //clean query buffer
		query << "SELECT curr,len FROM maglen WHERE calibration_name='" << maglen_calibration_name << "' ORDER BY curr";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=2 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on maglen");
		}
		int nrows=mysql_num_rows(queryresult);
		doubleVector curr;
		doubleVector len;
		//store table values into base vectors
		for( int i=0; i< nrows; i++ ){
			row=mysql_fetch_row(queryresult);
			curr.push_back(strtod(row[0],0));
			len.push_back(strtod(row[1],0));
		}
		//create Interpolator::Interpolator for field;
		length_spline = new Spline(curr,len);
		length_interpolator=length_spline;
		mysql_free_result(queryresult);
	}

	//------------------
	if(maglen_polynomial_name != "null"){ //create multipolynomial interpolators for the magnetic_len{
		//step 4b: detect and extract the  multi polynomial interpolators for the magneti length
		query.str(""); //clean query buffer
		query <<"SELECT min_direct,max_direct,degree,d0,d1,d2,d3,d4,d5,d6 FROM polynomial_calibration WHERE polynomial_name='" << maglen_polynomial_name <<"' ORDER BY min_direct";
		myres=mysql_query(&mysql,query.str().c_str());
		if(myres != 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		queryresult=mysql_store_result(&mysql);
		if(queryresult == 0){
			throw std::invalid_argument(mysql_error(&mysql));
		}
		num_fields=mysql_num_fields(queryresult);

		if(num_fields !=10 ){ //check that we get what we expected
			mysql_free_result(queryresult);
			throw std::range_error("wrong number of fields from SQl query on polynomial_calibration");
		}
		int nrows=mysql_num_rows(queryresult); //number of rows is the number of poynomials

		InterpolatingPolynoamialVector pvector; //vector of plynomials
		for( int i=0; i< nrows; i++ ){
			InterpolatingPolynomial poly; //temporary polynomial
			row=mysql_fetch_row(queryresult); //extrct data fromquer and insert in polynomial
			poly.xMin=strtod(row[0],0);
			poly.xMax=strtod(row[1],0);
			int degree=atoi(row[2]);
			for(int k=0; k<=degree;k++){
				poly.coefficient.push_back(strtod(row[k+3],0));
			}
			pvector.push_back(poly); //insert polynomial int vector;
		}
		//crete the multypolynomial
		length_multipoly=new multiPolynomial(pvector);
		length_interpolator=length_multipoly;
		mysql_free_result(queryresult);
	}

	//if neither a spline nor a multipolynomial are provided for magnetic length, use static data from MODEL
	// and create a 0-degree polynomial (that  is constant value).
	if(length_interpolator==0){
		//step 4b magnetic_length
		InterpolatingPolynomial poly; //temporary polynomial
		InterpolatingPolynoamialVector pvector; //vector of plynomials
		double min,max;
		field_interpolator->get_range(min,max);
		poly.xMin=min;
		poly.xMax=max;
		poly.coefficient.push_back(strtod(magnetic_length.c_str(),0));
		pvector.push_back(poly);
		length_multipoly=new multiPolynomial(pvector);
		length_interpolator=length_multipoly;

	}

	//step 5: create the solenoid and transfer ownership to caller
	Solenoid* solen= new Solenoid(focussing,*field_interpolator,*invfield_interpolator,*length_interpolator,solenname);

	solen->setGeometricLength(atoi(magnetic_length.c_str()));

	return solen;

} //newSolenoid

} // cloase namspace Field
