#include <iostream>
#include "dipole.h"
#include <multipolynomial.h>
#include <spline.h>
#include "magnetfactory.h"
#include <stdexcept>
#include <cmath>

using namespace std;
using namespace magnetfield;
using namespace Interpolator;

int main(){
	cout << "test of Field::Dipole classes" << endl;
	
	InterpolatingPolynomial p0,p1,p2;
	p0.xMin=0.5;
	p0.xMax=1.5;
	p0.coefficient.push_back(1.1);
	
	p1.xMin=1.5;
	p1.xMax=2.5;
	p1.coefficient.push_back(1.1);
	p1.coefficient.push_back(2.2);
	
	p2.xMin=2.5;
	p2.xMax=3.5;
	p2.coefficient.push_back(1.1);
	p2.coefficient.push_back(2.2);
	p2.coefficient.push_back(3.3);
	
	//stuff polynomial into vector
	
	InterpolatingPolynoamialVector pvector;
	pvector.push_back(p0);
	pvector.push_back(p1);
	pvector.push_back(p2);
	
	multiPolynomial *pfield = new multiPolynomial(pvector);
	
	doubleVector x,y;
	const int NP=30;
	for(int i=0; i < NP; i++){
		x.push_back(i*.02);
		y.push_back(i*i*.02);
	}
	Spline* invfield = new Spline(x,y);
	
	Interpolator::Interpolator *field,*inverse,*len;
	
	
	field=pfield;
	inverse=invfield;
	len=pfield;
	bool focus=true;
 	Dipole *mydip= new Dipole(focus,*field,*inverse,*len,*field,*inverse,"test","pstest");
	delete pfield; pfield=0;
	delete invfield; invfield=0;
	double currm,currM;
	mydip->getExcitationRange(currm,currM);
	
	cout << "min I:" << currm << " max I:" << currM << endl;
	{
		cout << "test strength ( )...";
		try {
			double k=mydip->excitationToStrength((currm+currM)/2.0, 1.0);
			cout << "k: " << k;
			k=mydip->excitationToStrength((currm+currM)/2.0, 0.0);
			cout << " ...FAIL,  NO Exception !"  << endl;
			return -1;
		}
		catch (std::range_error& ex){
			cout << " ...OK,  range error: " << ex.what() << endl;
		}
		catch(...) {
			cout << " ...FAIL,  unexpected exception!"  << endl;
			return -1;
		}
	
	}
	
	{
		cout << "test excitation(field)...";
		double gmin,gmax;
		try {
			double f=mydip->fieldToExcitation(gmin);
			cout << "f: " << f << " gmin:" <<gmin;
			mydip->getStrengthRange(gmin,gmax,2.0);
			f=mydip->strengthToExcitation(gmin, 2.0);
			cout << " f[2.0GeV]: " << f << " gmin[2.0GeV]:" << gmin;
			f=mydip->strengthToExcitation(gmin, 0.0);
			cout << " ...FAIL,  NO Exception !"  << endl;
		}
		catch (std::range_error& ex){
			cout << " ...OK,  range error: " << ex.what() << endl;
		}
		catch(...) {
			cout << " ...FAIL,  unexpected exception!"  << endl;
		}
		
	}
	
	
  	delete mydip;
	
	try{
		cout << "test Magnetfactory 1.. " ;
		Magnetfactory* mf1 =new Magnetfactory("fermi_magnets","magnetro","0gauss2","goofy.elettra.trieste.it"); delete mf1;
		cout << "test Magnetfactory 1 OK" << endl;
	}
	catch(...){
		cout << "test Magnetfactory 1 ...FAIL,  unexpected exception!"  << endl;
		return -1;
	}
	
	//test building of dipoles
	Dipole *d1=nullptr;
	try{
		cout << "test building of Dipole.. " ;
		Magnetfactory* mf1 =new Magnetfactory();
		d1=mf1->newDipole("B_S");
		delete mf1;
		cout << ".. OK" << endl;
		
	}
	catch (std::range_error& ex){
		cout << "test building of Dipole.. ...FAIL,  range_error: " << ex.what() << endl;
		return -1;
	}
	catch (std::invalid_argument& ex){
		cout << "test building of Dipole.. ...FAIL,  invalid_argument: " << ex.what() << endl;
		return -1;
	}
	catch(...){
		cout << "test building of Dipole.. ...FAIL,  unexpected exception!"  << endl;
		return -1;
	}
	
	//play wtih d1;
	double imin,imax;
	double emin,emax;
	double fmin,fmax;
	double smin,smax;
	
	d1->getExcitationRange(imin,imax);
	cout<< d1->getName() << " current range " << imin << " " << imax << endl;
	d1->getEnergyRange(emin,emax);
	cout<< d1->getName() << " energy range " << emin << " " << emax << endl;
	
	d1->getFieldRange(fmin,fmax);
	cout<< d1->getName() << " field range " << fmin << " " << fmax << endl;
	
	
	d1->getStrengthRange(smin,smax,2.0);
	cout<< d1->getName() << " strength range at 2.0 GeV " << smin << " " << smax << endl;
	d1->getCurvatureRange(smin,smax,2.0);
	cout<< d1->getName() << " radius range at 2.0 GeV " << 1.0/smin << " " << 1.0/smax << endl;
	
	int steps=10;
	
	
	double delta=(imax-imin)/steps;
	double mfield=NAN,strength=NAN,mlen=NAN,curr=NAN,energy=NAN;
	cout << "#curr\tmfield\tenergy\tmlen\tstrength  -- at 2.0 GeV" << endl;
	for(int i=0;i<=steps;i++){
		curr=imin+delta*i;
		mfield=d1->excitationToField(curr);
		mlen=d1->excitationToMagneticLength(curr);
		strength=d1->excitationToStrength(curr,2.0);
		energy=d1->excitationToEnergy(curr);
		cout << curr << "\t" << mfield << "\t" << energy << "\t"<< mlen << "\t" << strength << endl;
	}
	double st=d1->energyToExcitation(2.0);
	cout << "Nom curr for 2.0 GeV I: "  << st << " energy: " << d1->excitationToEnergy(st) <<  endl;
	cout << " MAx energy: " << d1->excitationToEnergy(1950.0) <<  endl;
	cout << "curr for 2.4695GeV: " << d1->energyToExcitation(2.4695) << endl;
	cout << d1->getName() << " angle: " << d1->getAngle() << " radius: " << d1->getRadius() << endl;
	
	//cout << d1->getName() << " gradient@500A: " << d1->getGradient(500.0) << " index@500A: " << d1->getIndex(500.0) << endl;
	
	if(d1) delete d1;
	d1=nullptr;
	{
		try{
			cout << "test building of Booster Dipole.. " ;
			Magnetfactory* mf1 =new Magnetfactory();
			d1=mf1->newDipole("B_B");
			delete mf1;
			cout << ".. OK" << endl;
			
		}
		catch (std::range_error& ex){
			cout << "test building of Booster Dipole.. ...FAIL,  range_error: " << ex.what() << endl;
			return -1;
		}
		catch (std::invalid_argument& ex){
			cout << "test building of Booster Dipole.. ...FAIL,  invalid_argument: " << ex.what() << endl;
			return -1;
		}
		catch(...){
			cout << "test building of Booster Dipole.. ...FAIL,  unexpected exception!"  << endl;
			return -1;
		}
		
	//play wtih d1;
// 		double imin,imax;
// 		double emin,emax;
// 		double fmin,fmax;
// 		double smin,smax;
		
		d1->getExcitationRange(imin,imax);
		cout<< d1->getName() << "booster dipole  current range " << imin << " " << imax << endl;
		d1->getEnergyRange(emin,emax);
		cout<< d1->getName() << "booster energy range " << emin << " " << emax << endl;
		
		d1->getFieldRange(fmin,fmax);
		cout<< d1->getName() << "booster dipole field range " << fmin << " " << fmax << endl;
		
		
		d1->getStrengthRange(smin,smax,2.0);
		cout<< d1->getName() << "booster dipole strength range at 2.0 GeV " << smin << " " << smax << endl;
		d1->getCurvatureRange(smin,smax,2.0);
		cout<< d1->getName() << "booster dipole radius range at 2.0 GeV " << 1.0/smin << " " << 1.0/smax << endl;
		
		steps=10;
		
		
		delta=(imax-imin)/steps;
		mfield=NAN;
		strength=NAN;
		mlen=NAN;
		curr=NAN;
		energy=NAN;
		cout << "#curr\tmfield\tenergy\tmlen\tstrength at 2.0 GeV " << endl;
		for(int i=0;i<=steps;i++){
			curr=imin+delta*i;
			if(curr < imin) curr=imin;
			if(curr > imax) curr=imax;
			mfield=d1->excitationToField(curr);
			mlen=d1->excitationToMagneticLength(curr);
			strength=d1->excitationToStrength(curr,2.0);
			energy=d1->excitationToEnergy(curr);
			cout << curr << "\t" << mfield << "\t" << energy << "\t"<< mlen << "\t" << strength << endl;
		}
		st=d1->energyToExcitation(0.05);
		cout << "Nom curr for 0.05 GeV I: "  << st << " energy: " << d1->excitationToEnergy(st) <<  endl;
		cout << " MAx energy: " << d1->excitationToEnergy(689.6) <<  endl;
		cout << "curr for 2.4695GeV: " << d1->energyToExcitation(2.5) << endl;
		cout << d1->getName() << " angle: " << d1->getAngle() << " radius: " << d1->getRadius() << endl;
		
		//check energy end strngth
		double Etest=0.114;
		st=d1->energyToExcitation(Etest);
		cout << "---- Etest: " << Etest <<" ------------"<<endl;
		cout << "Nom. curr. : "  << st << " energy: " << d1->excitationToEnergy(st) <<  endl;
		cout << "Deviation angle [rad]: "<<d1->excitationToStrength(st,Etest) << " Geometrical angle [deg]: " << d1->getAngle() << endl;
		st=d1->strengthToExcitation(0.224399475256,Etest);
		cout << "Nom. curr. for 224.399475256 [mrd] deviation :" << st << endl;
		cout << "Check for angle at this current : " << d1->excitationToStrength(st,Etest) <<endl;
		cout << "Check for energy at this angle: " << d1->excitationToEnergy(st) <<  endl;
	//cout << d1->getName() << " gradient@500A: " << d1->getGradient(500.0) << " index@500A: " << d1->getIndex(500.0) << endl;
	}
	if(d1) delete d1;
	d1=nullptr;
	cout << endl << endl << "-----------------------------" << endl;
	{
		try{
			cout << "test building of PTB SIB " ;
			Magnetfactory* mf1 =new Magnetfactory();
			d1=mf1->newDipole("SIB_PTB1.1");
			delete mf1;
			cout << ".. OK" << endl;
			
		}
		catch (std::range_error& ex){
			cout << "test building of SIB.. ...FAIL,  range_error: " << ex.what() << endl;
			return -1;
		}
		catch (std::invalid_argument& ex){
			cout << "test building of SIB.. ...FAIL,  invalid_argument: " << ex.what() << endl;
			return -1;
		}
		catch(...){
			cout << "test building of SIB.. ...FAIL,  unexpected exception!"  << endl;
			return -1;
		}
		
	//play wtih d1;
// 		double imin,imax;
// 		double emin,emax;
// 		double fmin,fmax;
// 		double smin,smax;
		
		d1->getExcitationRange(imin,imax);
		cout<< d1->getName() << "SIB  volt range " << imin << " " << imax << endl;
		d1->getEnergyRange(emin,emax);
		cout<< d1->getName() << "SIB energy range " << emin << " " << emax << endl;
		
		
		steps=20;
		
		imin=0.01;
		imax=0.6;
		delta=(imax-imin)/steps;
		mfield=NAN;
		strength=NAN;
		mlen=NAN;
		curr=NAN;
		energy=NAN;
		cout << "kvolt\tenergy" << endl;
		for(int i=0;i<=steps;i++){
			curr=imin+delta*i;
			energy=d1->excitationToEnergy(curr);
			cout << curr << "\t" << energy << endl;
		}
		st=d1->energyToExcitation(0.1);
		cout << "energy at 0.5kV: " << d1->excitationToEnergy(0.5) <<  endl;
		cout << "kvolt for 0.1GeV: " << d1->energyToExcitation(0.1) << endl;
		cout << "kvolt for 0.108GeV: " << d1->energyToExcitation(0.108) << endl;
		
	//cout << d1->getName() << " gradient@500A: " << d1->getGradient(500.0) << " index@500A: " << d1->getIndex(500.0) << endl;
	}
	if(d1) delete d1;
	
	
	
	d1=nullptr;
	{
		try{
			cout << "test building of Booster Dipole.. " ;
			Magnetfactory* mf1 =new Magnetfactory();
			d1=mf1->newDipole("B_B");
			delete mf1;
			cout << ".. OK" << endl;
			
		}
		catch (std::range_error& ex){
			cout << "test building of Booster Dipole.. ...FAIL,  range_error: " << ex.what() << endl;
			return -1;
		}
		catch (std::invalid_argument& ex){
			cout << "test building of Booster Dipole.. ...FAIL,  invalid_argument: " << ex.what() << endl;
			return -1;
		}
		catch(...){
			cout << "test building of Booster Dipole.. ...FAIL,  unexpected exception!"  << endl;
			return -1;
		}
		
	//play wtih d1;
// 		double imin,imax;
// 		double emin,emax;
// 		double fmin,fmax;
// 		double smin,smax;
		
		d1->getExcitationRange(imin,imax);
		cout<< d1->getName() << "booster dipole  current range " << imin << " " << imax << endl;
		d1->getEnergyRange(emin,emax);
		cout<< d1->getName() << "booster energy range " << emin << " " << emax << endl;
		
		d1->getFieldRange(fmin,fmax);
		cout<< d1->getName() << "booster dipole field range " << fmin << " " << fmax << endl;
		
		
		d1->getStrengthRange(smin,smax,2.0);
		cout<< d1->getName() << "booster dipole strength range at 2.0 GeV " << smin << " " << smax << endl;
		d1->getCurvatureRange(smin,smax,2.0);
		cout<< d1->getName() << "booster dipole radius range at 2.0 GeV " << 1.0/smin << " " << 1.0/smax << endl;
		
		steps=10;
		
		
		delta=(imax-imin)/steps;
		mfield=NAN;
		strength=NAN;
		mlen=NAN;
		curr=NAN;
		energy=NAN;
		cout << "#curr\tmfield\tenergy\tmlen\tstrength at 2.0 GeV " << endl;
		for(int i=0;i<=steps;i++){
			curr=imin+delta*i;
			if(curr < imin) curr=imin;
			if(curr > imax) curr=imax;
			mfield=d1->excitationToField(curr);
			mlen=d1->excitationToMagneticLength(curr);
			strength=d1->excitationToStrength(curr,2.0);
			energy=d1->excitationToEnergy(curr);
			cout << curr << "\t" << mfield << "\t" << energy << "\t"<< mlen << "\t" << strength << endl;
		}
		st=d1->energyToExcitation(0.05);
		cout << "Nom curr for 0.05 GeV I: "  << st << " energy: " << d1->excitationToEnergy(st) <<  endl;
		cout << " MAx energy: " << d1->excitationToEnergy(689.6) <<  endl;
		cout << "curr for 2.4695GeV: " << d1->energyToExcitation(2.5) << endl;
		cout << d1->getName() << " angle: " << d1->getAngle() << " radius: " << d1->getRadius() << endl;
		
		//check energy end strngth
		double Etest=0.114;
		st=d1->energyToExcitation(Etest);
		cout << "---- Etest: " << Etest <<" ------------"<<endl;
		cout << "Nom. curr. : "  << st << " energy: " << d1->excitationToEnergy(st) <<  endl;
		cout << "Deviation angle [rad]: "<<d1->excitationToStrength(st,Etest) << " Geometrical angle [deg]: " << d1->getAngle() << endl;
		st=d1->strengthToExcitation(0.224399475256,Etest);
		cout << "Nom. curr. for 0.224399475256 [rad] deviation :" << st << endl;
		cout << "Check for angle at this current : " << d1->excitationToStrength(st,Etest) <<endl;
		cout << "Check for energy at this angle: " << d1->excitationToEnergy(st) <<  endl;
	//cout << d1->getName() << " gradient@500A: " << d1->getGradient(500.0) << " index@500A: " << d1->getIndex(500.0) << endl;
	}
	if(d1) delete d1;
	d1=0;
	cout << endl << endl << "-----------------------------" << endl;
	{
		try{
			cout << "test building of PTB SIB " ;
			Magnetfactory* mf1 =new Magnetfactory();
			d1=mf1->newDipole("SIB_PTB1.1");
			delete mf1;
			cout << ".. OK" << endl;
			
		}
		catch (std::range_error& ex){
			cout << "test building of SIB.. ...FAIL,  range_error: " << ex.what() << endl;
			return -1;
		}
		catch (std::invalid_argument& ex){
			cout << "test building of SIB.. ...FAIL,  invalid_argument: " << ex.what() << endl;
			return -1;
		}
		catch(...){
			cout << "test building of SIB.. ...FAIL,  unexpected exception!"  << endl;
			return -1;
		}
		
	//play wtih d1;
// 		double imin,imax;
// 		double emin,emax;
// 		double fmin,fmax;
// 		double smin,smax;
		
		d1->getExcitationRange(imin,imax);
		cout<< d1->getName() << "SIB  volt range " << imin << " " << imax << endl;
		d1->getEnergyRange(emin,emax);
		cout<< d1->getName() << "SIB energy range " << emin << " " << emax << endl;
		
		
		steps=20;
		
		imin=0.01;
		imax=0.6;
		delta=(imax-imin)/steps;
		mfield=NAN;
		strength=NAN;
		mlen=NAN;
		curr=NAN;
		energy=NAN;
		cout << "kvolt\tenergy" << endl;
		for(int i=0;i<=steps;i++){
			curr=imin+delta*i;
			energy=d1->excitationToEnergy(curr);
			cout << curr << "\t" << energy << endl;
		}
		st=d1->energyToExcitation(0.1);
		cout << "energy at 0.5kV: " << d1->excitationToEnergy(0.5) <<  endl;
		cout << "kvolt for 0.1GeV: " << d1->energyToExcitation(0.1) << endl;
		cout << "kvolt for 0.108GeV: " << d1->energyToExcitation(0.108) << endl;
		
	//cout << d1->getName() << " gradient@500A: " << d1->getGradient(500.0) << " index@500A: " << d1->getIndex(500.0) << endl;
	}
	if(d1) delete d1;
	
	d1=nullptr;
	{
		try{
			cout << "test building of PTB Dipole.. " ;
			Magnetfactory* mf1 =new Magnetfactory();
			d1=mf1->newDipole("B_PTB1.1");
			delete mf1;
			cout << ".. OK" << endl;
			
		}
		catch (std::range_error& ex){
			cout << "test building of PTB Dipole.. ...FAIL,  range_error: " << ex.what() << endl;
			return -1;
		}
		catch (std::invalid_argument& ex){
			cout << "test building of PTB Dipole.. ...FAIL,  invalid_argument: " << ex.what() << endl;
			return -1;
		}
		catch(...){
			cout << "test building of PTB Dipole.. ...FAIL,  unexpected exception!"  << endl;
			return -1;
		}
		
	//play wtih d1;
// 		double imin,imax;
// 		double emin,emax;
// 		double fmin,fmax;
// 		double smin,smax;
		
		d1->getExcitationRange(imin,imax);
		cout<< d1->getName() << "ptb dipole  current range " << imin << " " << imax << endl;
		d1->getEnergyRange(emin,emax);
		cout<< d1->getName() << "ptb energy range " << emin << " " << emax << endl;
		
		d1->getFieldRange(fmin,fmax);
		cout<< d1->getName() << "ptb dipole field range " << fmin << " " << fmax << endl;
		
		
		d1->getStrengthRange(smin,smax,0.104);
		cout<< d1->getName() << "ptb dipole strength range at 0.104 GeV " << smin << " " << smax << endl;
		d1->getCurvatureRange(smin,smax,2.0);
		cout<< d1->getName() << "ptb dipole radius range at 0.104 GeV " << 1.0/smin << " " << 1.0/smax << endl;
		
		steps=10;
		
		
		delta=(imax-imin)/steps;
		mfield=NAN;
		strength=NAN;
		mlen=NAN;
		curr=NAN;
		energy=NAN;
		cout << "#curr\tmfield\tenergy\tmlen\tstrength at 0.104 GeV " << endl;
		for(int i=0;i<=steps;i++){
			curr=imin+delta*i;
			if(curr < imin) curr=imin;
			if(curr > imax) curr=imax;
			mfield=d1->excitationToField(curr);
			mlen=d1->excitationToMagneticLength(curr);
			strength=d1->excitationToStrength(curr,2.0);
			energy=d1->excitationToEnergy(curr);
			cout << curr << "\t" << mfield << "\t" << energy << "\t"<< mlen << "\t" << strength << endl;
		}
		st=d1->energyToExcitation(0.05);
		cout << "Nom curr for 0.05 GeV I: "  << st << " energy: " << d1->excitationToEnergy(st) <<  endl;
		cout << " MAx energy: " << d1->excitationToEnergy(180.0) <<  endl;
		
		cout << d1->getName() << " angle: " << d1->getAngle() << " radius: " << d1->getRadius() << endl;
		
		//check energy end strngth
		double Etest=0.104;
		st=d1->energyToExcitation(Etest);
		cout << "---- Etest: " << Etest <<" ------------"<<endl;
		cout << "Nom. curr. : "  << st << " energy: " << d1->excitationToEnergy(st) <<  endl;
		cout << "Deviation angle [rad]: "<<d1->excitationToStrength(st,Etest) << " Geometrical angle [deg]: " << d1->getAngle() << endl;
		st=d1->strengthToExcitation(0.2617993877991494365,Etest);
		cout << "Nom. curr. for 0.2617993877991494365 [rad] deviation :" << st << endl;
		cout << "Check for angle at this current : " << d1->excitationToStrength(st,Etest) <<endl;
		cout << "Check for energy at this angle: " << d1->excitationToEnergy(st) <<  endl;
	//cout << d1->getName() << " gradient@500A: " << d1->getGradient(500.0) << " index@500A: " << d1->getIndex(500.0) << endl;
	}
	if(d1) delete d1;


d1=nullptr;
    {
        try{
            cout << "test building of B_S Dipole.. " ;
            Magnetfactory* mf1 =new Magnetfactory("fermi_magnets","magnetro","0gauss2","goofy");
            d1=mf1->newDipole("B_S");
            delete mf1;
            cout << ".. OK" << endl;

        }
        catch (std::range_error& ex){
            cout << "test building of B_S1.1 Dipole.. ...FAIL,  range_error: " << ex.what() << endl;
            return -1;
        }
        catch (std::invalid_argument& ex){
            cout << "test building of B_S1.1 Dipole.. ...FAIL,  invalid_argument: " << ex.what() << endl;
            return -1;
        }
        catch(...){
            cout << "test building of B_S1.1 Dipole.. ...FAIL,  unexpected exception!"  << endl;
            return -1;
        }

    //play wtih d1;
//      double imin,imax;
//      double emin,emax;
//      double fmin,fmax;
//      double smin,smax;

        d1->getExcitationRange(imin,imax);
        cout<< d1->getName() << " dipole  current range " << imin << " " << imax << endl;
        d1->getEnergyRange(emin,emax);
        cout<< d1->getName() << " energy range " << emin << " " << emax << endl;

        d1->getFieldRange(fmin,fmax);
        cout<< d1->getName() << " dipole field range " << fmin << " " << fmax << endl;
        cout<< d1->getPsName() << " dipole power supply";




        steps=10;


        delta=(imax-imin)/steps;
        mfield=NAN;
        strength=NAN;
        mlen=NAN;
        curr=NAN;
        energy=NAN;
        cout << "#curr\tmfield\tenergy\tmlen\tstrength" << endl;
        for(int i=0;i<=steps;i++){
            curr=imin+delta*i;
            if(curr < imin) curr=imin;
            if(curr > imax) curr=imax;
            mfield=d1->excitationToField(curr);
            mlen=d1->excitationToMagneticLength(curr);
            strength=d1->excitationToStrength(curr,2.0);
            energy=d1->excitationToEnergy(curr);
            cout << curr << "\t" << mfield << "\t" << energy << "\t"<< mlen << "\t" << strength << endl;
        }


        //check energy end excitation
        double Etest=2.00;
        //double Itest = 1419.920;
        double Itest = 1845.096617;
        st=d1->energyToExcitation(Etest);
        cout << "---- Etest: " << Etest <<" ------------"<<endl;
        cout << "Nom. curr. : "  << st << " energy: " << d1->excitationToEnergy(st) <<  endl;
        cout << "Deviation angle [rad]: "<<d1->excitationToStrength(st,Etest) << " Geometrical angle [deg]: " << d1->getAngle() << endl;

        cout << "Energy at standard operating current: "<< d1->excitationToEnergy(Itest) <<  endl;
    }
    if(d1) delete d1;

	return 0;
}
