#include <iostream>
#include "kicker.h"
#include <multipolynomial.h>
#include <spline.h>
#include "magnetfactory.h"
#include <stdexcept>

using namespace std;
using namespace magnetfield;
using namespace Interpolator;

int main(){
	cout << "test of Field::Kicker classes" << endl;
	
	InterpolatingPolynomial p0,p1,p2;
	p0.xMin=0.5;
	p0.xMax=1.5;
	p0.coefficient.push_back(1.1);
	
	p1.xMin=1.5;
	p1.xMax=2.5;
	p1.coefficient.push_back(1.1);
	p1.coefficient.push_back(2.2);
	
	p2.xMin=2.5;
	p2.xMax=3.5;
	p2.coefficient.push_back(1.1);
	p2.coefficient.push_back(2.2);
	p2.coefficient.push_back(3.3);
	
	//stuff polynomial into vector
	
	InterpolatingPolynoamialVector pvector;
	pvector.push_back(p0);
	pvector.push_back(p1);
	pvector.push_back(p2);
	
	multiPolynomial *pfield = new multiPolynomial(pvector);
	
	doubleVector x,y;
	const int NP=30;
	for(int i=0; i < NP; i++){
		x.push_back(i*.02);
		y.push_back(i*i*.02);
	}
	Spline* invfield = new Spline(x,y);
	
	Interpolator::Interpolator *field,*inverse,*len;
	
	
	field=pfield;
	inverse=invfield;
	len=pfield;
	bool focus=true;
 	Kicker*myq= new Kicker(focus,*field,*inverse,*len,"test","pstest");
	
	double pv=pfield->evaluate(1.1);
	delete pfield; pfield=0;
	delete invfield; invfield=0;
	double currm,currM;
	myq->getExcitationRange(currm,currM);
	
	cout << "min I:" << currm << " max I:" << currM << endl;
	double v=myq->excitationToField((currm+currM)/2.0);
	
	double gmin,gmax;
	myq->getFieldRange(gmin,gmax);
	cout << "min g:" << gmin << " max g:" << gmax << endl;
	double ex=myq->fieldToExcitation(gmin);
	double le=myq->excitationToMagneticLength((currm+currM)/2.0);
	cout << v << " " << pv << " " << ex << " " << le << endl;
	
	{
		cout << "test strength ( )...";
		try {
			double k=myq->excitationToStrength((currm+currM)/2.0, 1.0);
			cout << "k: " << k;
			k=myq->excitationToStrength((currm+currM)/2.0, 0.0);
			cout << " ...FAIL,  NO Exception !"  << endl;
		}
		catch (std::range_error& e){
			cout << " ...OK,  range error: " << e.what() << endl;
		}
		catch(...) {
			cout << " ...FAIL,  unexpected exception!"  << endl;
		}
	
	}
	
	{
		cout << "test excitation(field)...";
		try {
			double f=myq->fieldToExcitation(gmin);
			cout << "f: " << f << " gmin:" <<gmin;
			myq->getStrengthRange(gmin,gmax,2.0);
			f=myq->strengthToExcitation(gmin, 2.0);
			cout << " f[2.0GeV]: " << f << " gmin[2.0GeV]:" << gmin;
			f=myq->strengthToExcitation(gmin, 0.0);
			cout << " ...FAIL,  NO Exception !"  << endl;
		}
		catch (std::range_error& e){
			cout << " ...OK,  range error: " << e.what() << endl;
		}
		catch(...) {
			cout << " ...FAIL,  unexpected exception!"  << endl;
		}
		
	}
	
	
  	delete myq;
	
	//test building of kicker
	Kicker*q1=nullptr;
	try{
		cout << "test building of Kicker.. " ;
		Magnetfactory* mf1 =new Magnetfactory();
		q1=mf1->newKicker("CV_S1.1");
		delete mf1;
		cout << " . OK! " << endl;
	}
	catch (std::range_error& e){
		cout << "test building of Kicker.. ...FAIL,  range_error: " << e.what() << endl;
	}
	catch (std::invalid_argument& e){
		cout << "test building of Kicker.. ...FAIL,  invalid_argument: " << e.what() << endl;
	}
	catch(...){
		cout << "test building of Kicker.. ...FAIL,  unexpected exception!"  << endl;
	}
	
	//play wtih q1;
	double imin,imax;
	
	q1->getFieldRange(imin,imax);
	cout<< q1->getName() << " field range " << imin << " " << imax << endl;
	
	q1->getStrengthRange(imin,imax,2.0);
	cout<< q1->getName() << " strength range@2.0GeV " << imin << " " << imax << endl;
	
	q1->getExcitationRange(imin,imax);
	cout<< q1->getName() << " current range " << imin << " " << imax << endl;
	
	int steps=100;
	double delta=(imax-imin)/steps;
	double mfield,strength,mlen,curr;
	cout << "#curr\tmfield\tstrength\tmlen" << endl;
	for(int i=0;i<=steps;i++){
		curr=imin+delta*i;
		mfield=q1->excitationToField(curr);
		mlen=q1->excitationToMagneticLength(curr);
		strength=q1->excitationToStrength(curr,2.0);
		cout << curr << "\t" << mfield << "\t" << strength << "\t"<< mlen << endl;
	}
	double st=q1->excitationToStrength(2.0,2.0);
	cout << "at nom curr (2.0) for 2.0GeV- field: " << q1->excitationToField(2.0) << " strength:" << st << endl;
	
	cout << q1->getName() << " " << q1->getPsName() << " nom curr at 2GeV (inverse calc) " << q1->strengthToExcitation(st,2.0) << endl;
	
	
	
	if(q1){
		delete q1;
		q1=nullptr;
	}
	
	
	{
		//test building of kicker for PTB
		
		try{
			cout << "\n\n\ntest building of PTB kicker.. " ;
			Magnetfactory* mf1 =new Magnetfactory();
			q1=mf1->newKicker("CV_PTB1.3");
			delete mf1;
			cout << " . OK! " << endl;
		}
		catch (std::range_error& e){
			cout << "test building of Kicker.. ...FAIL,  range_error: " << e.what() << endl;
		}
		catch (std::invalid_argument& e){
			cout << "test building of Kicker.. ...FAIL,  invalid_argument: " << e.what() << endl;
		}
		catch(...){
			cout << "test building of Kicker.. ...FAIL,  unexpected exception!"  << endl;
		}
		
	//play wtih q1;
				
		
		q1->getFieldRange(imin,imax);
		cout<< q1->getName() << " field range " << imin << " " << imax << endl;
		
		q1->getStrengthRange(imin,imax,0.1);
		cout<< q1->getName() << " strength range@0.1GeV " << imin << " " << imax << endl;
		
		q1->getExcitationRange(imin,imax);
		cout<< q1->getName() << " current range " << imin << " " << imax << endl;
		
		steps=100;
		delta=(imax-imin)/steps;
				
		cout << "#curr\tmfield\tstrength\tmlen" << endl;
		for(int i=0;i<=steps;i++){
			curr=imin+delta*i;
			mfield=q1->excitationToField(curr);
			mlen=q1->excitationToMagneticLength(curr);
			strength=q1->excitationToStrength(curr,0.1);
			cout << curr << "\t" << mfield << "\t" << strength << "\t"<< mlen << endl;
		}
		st=q1->excitationToStrength(1.0,0.1);
		cout << "at nom curr (1.0) for 0.1GeV- field: " << q1->excitationToField(1.0) << " strength:" << st << endl;
		
		cout << q1->getName() << " " << q1->getPsName() << " nom curr at 0.1GeV (inverse calc) " << q1->strengthToExcitation(st,0.1) << endl;
		
		
		
		if(q1) delete q1;
	}
	
	{
		//test building of kicker for PTB
		
		try{
			cout << "\n\n\ntest building of Booster kicker.. " ;
			Magnetfactory* mf1 =new Magnetfactory();
			q1=mf1->newKicker("CH_B");
			delete mf1;
			cout << " . OK! " << endl;
		}
		catch (std::range_error& e){
			cout << "test building of Kicker.. ...FAIL,  range_error: " << e.what() << endl;
		}
		catch (std::invalid_argument& e){
			cout << "test building of Kicker.. ...FAIL,  invalid_argument: " << e.what() << endl;
		}
		catch(...){
			cout << "test building of Kicker.. ...FAIL,  unexpected exception!"  << endl;
		}
		
	//play wtih q1;
		
		q1->getFieldRange(imin,imax);
		cout<< q1->getName() << " field range " << imin << " " << imax << endl;
		
		q1->getStrengthRange(imin,imax,2);
		cout<< q1->getName() << " strength range@2GeV " << imin << " " << imax << endl;
		
		q1->getExcitationRange(imin,imax);
		cout<< q1->getName() << " current range " << imin << " " << imax << endl;
		
		steps=100;
		delta=(imax-imin)/steps;
		
		cout << "#curr\tmfield\tstrength\tmlen" << endl;
		for(int i=0;i<=steps;i++){
			curr=imin+delta*i;
			mfield=q1->excitationToField(curr);
			mlen=q1->excitationToMagneticLength(curr);
			strength=q1->excitationToStrength(curr,2.0);
			cout << curr << "\t" << mfield << "\t" << strength << "\t"<< mlen << endl;
		}
		st=q1->excitationToStrength(1.0,0.1);
		cout << "at nom curr (1.0) for 0.1GeV- field: " << q1->excitationToField(1.0) << " strength:" << st << endl;
		
		cout << q1->getName() << " " << q1->getPsName() << " nom curr at 0.1GeV (inverse calc) " << q1->strengthToExcitation(st,0.1) << endl;
		
		
		
		if(q1) delete q1;
	}
	{
		//test building of kicker for PTB
		
		try{
			cout << "\n\n\ntest building of Booster KIB " ;
			Magnetfactory* mf1 =new Magnetfactory();
			q1=mf1->newKicker("KIB_B28.1");
			delete mf1;
			cout << " . OK! " << endl;
		}
		catch (std::range_error& e){
			cout << "test building of Kicker.. ...FAIL,  range_error: " << e.what() << endl;
		}
		catch (std::invalid_argument& e){
			cout << "test building of Kicker.. ...FAIL,  invalid_argument: " << e.what() << endl;
		}
		catch(...){
			cout << "test building of Kicker.. ...FAIL,  unexpected exception!"  << endl;
		}
		
	//play wtih q1;

	
		q1->getStrengthRange(imin,imax,0.1);
		cout<< q1->getName() << " strength range@0.1GeV " << imin << " " << imax << endl;

		q1->getStrengthRange(imin,imax,0.108);
		cout<< q1->getName() << " strength range@0.108GeV " << imin << " " << imax << endl;
		
		q1->getExcitationRange(imin,imax);
		cout<< q1->getName() << " kvolt range " << imin << " " << imax << endl;
		
		steps=20;
		delta=(imax-imin)/steps;

		cout << "#kvolt\tstrength [rad] @100MeV" << endl;
		for(int i=0;i<=steps;i++){
			curr=imin+delta*i;
			strength=q1->excitationToStrength(curr,0.1);
			cout << curr << "\t" << strength << endl;
		}

		steps=20;
		double emax=0.120;
		double emin=0.08;
		delta=(emax-emin)/steps;
		double energy;
		double volt;
		strength=4.5e-3; //nominal kick for injection
		cout << "#kvolt\tenergy for 4.5mrad" << endl;
		for(int i=0;i<=steps;i++){
			energy=emin+delta*i;
			volt=q1->strengthToExcitation(strength,energy);
			cout << volt << "\t" << energy << "\t" << endl;
		}
		
		
		if(q1) delete q1;
	}

	{
		//test building of KEB bor fooster extr
		
		try{
			cout << "\n\n\ntest building of Booster KEB " ;
			Magnetfactory* mf1 =new Magnetfactory();
			q1=mf1->newKicker("KEB_B13.1");
			delete mf1;
			cout << " . OK! " << endl;
		}
		catch (std::range_error& e){
			cout << "test building of Kicker.. ...FAIL,  range_error: " << e.what() << endl;
		}
		catch (std::invalid_argument& e){
			cout << "test building of Kicker.. ...FAIL,  invalid_argument: " << e.what() << endl;
		}
		catch(...){
			cout << "test building of Kicker.. ...FAIL,  unexpected exception!"  << endl;
		}
		
	//play wtih q1;
		
		q1->getStrengthRange(imin,imax,0.5);
		cout<< q1->getName() << " strength range@0.5GeV " << imin << " " << imax << endl;
		
		q1->getStrengthRange(imin,imax,0.588);
		cout<< q1->getName() << " strength range@0.588GeV " << imin << " " << imax << endl;
		
		q1->getExcitationRange(imin,imax);
		cout<< q1->getName() << " kvolt range " << imin << " " << imax << endl;
		
		steps=20;
		delta=(imax-imin)/steps;

		cout << "#kvolt\tstrength [rad] @500eV" << endl;
		for(int i=0;i<=steps;i++){
			curr=imin+delta*i;
			strength=q1->excitationToStrength(curr,0.5);
			cout << curr << "\t" << strength << endl;
		}
		
		steps=20;
		double emax=0.7;
		double emin=0.500;
		delta=(emax-emin)/steps;
		double energy;
		double volt;
		strength=2.5; //nominal kick for extraction
		cout << "#kvolt\tenergy for 2.5 mrad" << endl;
		for(int i=0;i<=steps;i++){
			energy=emin+delta*i;
			volt=q1->strengthToExcitation(strength,energy);
			cout << volt << "\t" << energy << "\t" << endl;
		}
		
		
		if(q1) delete q1;
	}
	
	{
		//test empirical correction factor
		
		try{
			cout << "\n\n\ntest empirical correction factor.. " ;
			Magnetfactory* mf1 =new Magnetfactory("fermi_magnets", "magnetro","0gauss2","localhost");
			q1=mf1->newKicker("CH_B");
			delete mf1;
			cout << " . OK! " << endl;
		}
		catch (std::range_error& e){
			cout << "test building of Kicker.. ...FAIL,  range_error: " << e.what() << endl;
		}
		catch (std::invalid_argument& e){
			cout << "test building of Kicker.. ...FAIL,  invalid_argument: " << e.what() << endl;
		}
		catch(...){
			cout << "test building of Kicker.. ...FAIL,  unexpected exception!"  << endl;
		}
		
	//play wtih q1;
		
		
		
		q1->getStrengthRange(imin,imax,2);
		cout<< q1->getName() << " strength range@2GeV alpha=1.0 " << imin << " " << imax << endl;
		
		q1->setAlpha(0.5);
		
		q1->getStrengthRange(imin,imax,2);
		cout<< q1->getName() << " strength range@2GeV alpha=0.5 " << imin << " " << imax << endl;
		
		
		q1->setAlpha(1.0);
		double strength2;
		cout << "#curr\t\tstrength1\tstrength2" << endl;
		for(int i=0;i<=steps;i++){
			curr=imin+delta*i;
			mfield=q1->excitationToField(curr);
			
			q1->setAlpha(1.0);
			strength=q1->excitationToStrength(curr,2.0);
			q1->setAlpha(0.5);
			strength2=q1->excitationToStrength(curr,2.0);
			cout << curr << "\t"  << strength << "\t"<< strength2 << endl;
		}
		q1->setAlpha(0.5);
		st=q1->excitationToStrength(1.0,0.1);
		cout << "at nom curr (1.0) for 0.1GeV- field: " << q1->excitationToField(1.0) << " strength:" << st << endl;
		
		cout << q1->getName() << " " << q1->getPsName() << " nom curr at 0.1GeV (inverse calc) " << q1->strengthToExcitation(st,0.1) << endl;
		
		
		
		if(q1) delete q1;
	}
	
	return 0;
}
