#include <iostream>
#include "quadrupole.h"
#include <interpolator.h>
#include <multipolynomial.h>
#include <spline.h>
#include "magnetfactory.h"
#include <stdexcept>

using namespace std;
using namespace magnetfield;
using namespace Interpolator;

int main(){
	cout << "test of Field::Magnet classes" << endl;

	InterpolatingPolynomial p0,p1,p2;
	p0.xMin=0.5;
	p0.xMax=1.5;
	p0.coefficient.push_back(1.1);

	p1.xMin=1.5;
	p1.xMax=2.5;
	p1.coefficient.push_back(1.1);
	p1.coefficient.push_back(2.2);

	p2.xMin=2.5;
	p2.xMax=3.5;
	p2.coefficient.push_back(1.1);
	p2.coefficient.push_back(2.2);
	p2.coefficient.push_back(3.3);

	//stuff polynomial into vector

	InterpolatingPolynoamialVector pvector;
	pvector.push_back(p0);
	pvector.push_back(p1);
	pvector.push_back(p2);

	multiPolynomial *pfield = new multiPolynomial(pvector);

	doubleVector x,y;
	const int NP=30;
	for(int i=0; i < NP; i++){
		x.push_back(i*.02);
		y.push_back(i*i*.02);
	}
	Spline* invfield = new Spline(x,y);

	Interpolator::Interpolator *field,*inverse,*len;


	field=pfield;
	inverse=invfield;
	len=pfield;
	bool focus=true;
 	Quadrupole *myq= new Quadrupole(focus,*field,*inverse,*len,"test","pstest");

	double pv=pfield->evaluate(1.1);
	delete pfield; pfield=nullptr;
	delete invfield; invfield=nullptr;
	double currm,currM;
	myq->getExcitationRange(currm,currM);

	cout << "min I:" << currm << "  max I:" << currM << endl;
	double v=myq->excitationToField((currm+currM)/2.0);

	double gmin,gmax;
	myq->getFieldRange(gmin,gmax);
	cout << "min g:" << gmin << "  max g:" << gmax << endl;
	double ex=myq->fieldToExcitation(gmin);
	double le=myq->excitationToMagneticLength((currm+currM)/2.0);
	cout << v << " " << pv << " " << ex << " " << le << endl;

	{
		cout << "test strength ( )...";
		try {
			double k=myq->excitationToStrength((currm+currM)/2.0, 1.0);
			cout << "k: " << k;
			k=myq->excitationToStrength((currm+currM)/2.0, 0.0);
			cout << " ...FAIL,  NO Exception !"  << endl;
		}
		catch (std::range_error& e){
			cout << " ...OK->  range error: " << e.what() << endl;
		}
		catch(...) {
			cout << " ...FAIL,  unexpected exception!"  << endl;
		}

	}

	{
		cout << "test excitation(field)...";
		try {
			double f=myq->fieldToExcitation(gmin);
			cout << "f: " << f << " gmin:" <<gmin;
			myq->getStrengthRange(gmin,gmax,2.0);
			f=myq->strengthToExcitation(gmin, 2.0);
			cout << " f[2.0GeV]: " << f << " gmin[2.0GeV]:" << gmin;
			f=myq->strengthToExcitation(gmin, 0.0);
			cout << " ...FAIL,  NO Exception !"  << endl;
		}
		catch (std::range_error& e){
			cout << " ...OK->  range error: " << e.what() << endl;
		}
		catch(...) {
			cout << " ...FAIL,  unexpected exception!"  << endl;
		}

	}


  	delete myq;

	try{
		cout << "test Magnetfactory 1: with explicit arguments...  " ;
		Magnetfactory* mf1 =new Magnetfactory("fermi_magnets", "magnetro","0gauss2","goofy"); delete mf1;
		cout << "test Magnetfactory 1 OK" << endl;
	}
	catch(...){
		cout << "test Magnetfactory 1 ...FAIL,  unexpected exception!"  << endl;
	}

	try{
		cout << "test Magnetfactory 2: with env variable ... " ;
		Magnetfactory* mf1 =new Magnetfactory(); delete mf1;
		cout << "test Magnetfactory 2 OK" << endl;
	}
	catch (std::invalid_argument& e){
		cout << "test Magnetfactory 2 FAIL->  invalid_argument: " << e.what() << endl;
	}
	catch(...){
		cout << "test Magnetfactory 2 ...FAIL,  unexpected exception!"  << endl;
	}

	//test building of qudrupole
	Quadrupole *q1=nullptr;
	try{
		cout << "test building of Quadrupole.. " ;
		//setFocSignConvention(foc_sign_vicky);
		Magnetfactory* mf1 =new Magnetfactory("fermi_magnets","magnetro","0gauss2","goofy");
		q1=mf1->newQuadrupole("Q1_S1.1");
		delete mf1;

	}
	catch (std::range_error& e){
		cout << "test building of Quadrupole.. ...FAIL,  range_error: " << e.what() << endl;
	}
	catch (std::invalid_argument& e){
		cout << "test building of Quadrupole.. ...FAIL,  invalid_argument: " << e.what() << endl;
	}
	catch(...){
		cout << "test building of Quadrupole.. ...FAIL,  unexpected exception!"  << endl;
	}

	//play wtih q1;
	double imin,imax;
	q1->getExcitationRange(imin,imax);
	cout<< "Q1_S1.1 current range " << imin << " " << imax << endl;
	int steps=100;
	double delta=(imax-imin)/steps;
	double mfield,strength,mlen,curr;
	for(int i=0;i<=steps;i++){
		curr=imin+delta*i;
		mfield=q1->excitationToField(curr);
		mlen=q1->excitationToMagneticLength(curr);
		strength=q1->excitationToStrength(curr,2.0);
		cout << curr << "\t" << mfield << "\t" << strength << "\t"<< mlen << endl;
	}
	double st=q1->excitationToStrength(173.593911,2.0);
	cout << "at nom curr (173.593911) for 2.0GeV: " << q1->excitationToField(173.593911) << " " << st << endl;
	/*st=1.99874; *///nominal strength according to F.I.
	cout << q1->getName() << " " << q1->getPsName() << " nom curr at 2GeV (inverse calc) " << q1->strengthToExcitation(st,2.0) << endl;



	if(q1) delete q1;
	return 0;
}
