#include <iostream>
#include "sextupole.h"

#include <multipolynomial.h>
#include <spline.h>
#include "magnetfactory.h"
#include <stdexcept>

using namespace std;
using namespace magnetfield;
using namespace Interpolator;

int main(){
	cout << "test of Field::Sextupole classes" << endl;
	
	InterpolatingPolynomial p0,p1,p2;
	p0.xMin=0.5;
	p0.xMax=1.5;
	p0.coefficient.push_back(1.1);
	
	p1.xMin=1.5;
	p1.xMax=2.5;
	p1.coefficient.push_back(1.1);
	p1.coefficient.push_back(2.2);
	
	p2.xMin=2.5;
	p2.xMax=3.5;
	p2.coefficient.push_back(1.1);
	p2.coefficient.push_back(2.2);
	p2.coefficient.push_back(3.3);
	
	//stuff polynomial into vector
	
	InterpolatingPolynoamialVector pvector;
	pvector.push_back(p0);
	pvector.push_back(p1);
	pvector.push_back(p2);
	
	multiPolynomial *pfield = new multiPolynomial(pvector);
	
	doubleVector x,y;
	const int NP=30;
	for(int i=0; i < NP; i++){
		x.push_back(i*.02);
		y.push_back(i*i*.02);
	}
	Spline* invfield = new Spline(x,y);
	
	Interpolator::Interpolator *field,*inverse,*len;
	
	
	field=pfield;
	inverse=invfield;
	len=pfield;
	bool focus=true;
 	Sextupole*myq= new Sextupole(focus,*field,*inverse,*len,"test","pstest");
	
	double pv=pfield->evaluate(1.1);
	delete pfield; pfield=nullptr;
	delete invfield; invfield=nullptr;
	double currm,currM;
	myq->getExcitationRange(currm,currM);
	
	cout << "min I:" << currm << " max I:" << currM << endl;
	double v=myq->excitationToField((currm+currM)/2.0);
	
	double gmin,gmax;
	myq->getFieldRange(gmin,gmax);
	cout << "min g:" << gmin << " max g:" << gmax << endl;
	double ex=myq->fieldToExcitation(gmin);
	double le=myq->excitationToMagneticLength((currm+currM)/2.0);
	cout << v << " " << pv << " " << ex << " " << le << endl;
	
	{
		cout << "test strength ( )...";
		try {
			double k=myq->excitationToStrength((currm+currM)/2.0, 1.0);
			cout << "k: " << k;
			k=myq->excitationToStrength((currm+currM)/2.0, 0.0);
			cout << " ...FAIL,  NO Exception !"  << endl;
		}
		catch (std::range_error& e){
			cout << " ...OK,  range error: " << e.what() << endl;
		}
		catch(...) {
			cout << " ...FAIL,  unexpected exception!"  << endl;
		}
	
	}
	
	{
		cout << "test excitation(field)...";
		try {
			double f=myq->fieldToExcitation(gmin);
			cout << "f: " << f << " gmin:" <<gmin;
			myq->getStrengthRange(gmin,gmax,2.0);
			f=myq->strengthToExcitation(gmin, 2.0);
			cout << " f[2.0GeV]: " << f << " gmin[2.0GeV]:" << gmin;
			f=myq->strengthToExcitation(gmin, 0.0);
			cout << " ...FAIL,  NO Exception !"  << endl;
		}
		catch (std::range_error& e){
			cout << " ...OK,  range error: " << e.what() << endl;
		}
		catch(...) {
			cout << " ...FAIL,  unexpected exception!"  << endl;
		}
		
	}
	
	
  	delete myq;
	
	try{
		cout << "test Magnetfactory 1.. " ;
		Magnetfactory* mf1 =new Magnetfactory("fermi_magnets", "magnetro","0gauss2","goofy"); delete mf1;
		cout << "test Magnetfactory 1 OK" << endl;
	}
	catch(...){
		cout << "test Magnetfactory 1 ...FAIL,  unexpected exception!"  << endl;
	}
	
	//test building of sextupole
	
	try{
		Sextupole*q1=nullptr;
		cout << "test building of Sextupole.. " ;
		//setFocSignConvention(foc_sign_vicky);
		Magnetfactory* mf1 =new Magnetfactory("fermi_magnets", "magnetro", "0gauss2", "goofy");
		q1=mf1->newSextupole("SF_B");
		delete mf1;
		cout << " . OK! " << endl;
	
	
	//play wtih q1;
		double imin,imax;
	
		q1->getFieldRange(imin,imax);
		cout<< q1->getName() << " field range " << imin << " " << imax << endl;
	
		q1->getStrengthRange(imin,imax,2.0);
		cout<< q1->getName() << " strength range@2.0GeV " << imin << " " << imax << endl;
	
		q1->getExcitationRange(imin,imax);
		cout<< q1->getName() << " current range " << imin << " " << imax << endl;
	
		int steps=100;
		double delta=(imax-imin)/steps;
		double mfield,strength,mlen,curr;
		cout << "#curr\tmfield\tstrength\tmlen" << endl;
		for(int i=0;i<=steps;i++){
			curr=imin+delta*i;
			mfield=q1->excitationToField(curr);
			mlen=q1->excitationToMagneticLength(curr);
			strength=q1->excitationToStrength(curr,2.0);
			cout << curr << "\t" << mfield << "\t" << strength << "\t"<< mlen << endl;
		}
		double st=q1->excitationToStrength(33.0,2.0);
		cout << "SF_B at nom curr (33.0) for 2.0GeV- field: " << q1->excitationToField(33.0) << " strength:" << st << endl;
	
		cout << q1->getName() << " " << "SF_B nom curr at 2GeV (inverse calc) " << q1->strengthToExcitation(st,2.0) << endl;
	
	
	
		if(q1) delete q1;
	}
	catch (std::range_error& e){
			cout << "test building of Sextupole SF_B.. ...FAIL,  range_error: " << e.what() << endl;
		}
 	catch (std::invalid_argument& e){
 		cout << "test building of Sextupole SF_B.. ...FAIL,  invalid_argument: " << e.what() << endl;
 	}
 	catch(...){
 		cout << "test building of Sextupole SF_B.. ...FAIL,  unexpected exception!"  << endl;
 	}
	
/*	
	try{
		Sextupole*q1=nullptr;
		cout << "test building of Sextupole.. " ;
		Magnetfactory* mf1 =new Magnetfactory("fermi_magnets", "magnet","0gauss2","goofy.elettra.trieste.it");
		q1=mf1->newSextupole("S1_S");
		delete mf1;
		cout << " . OK! " << endl;
		
		
	//play wtih q1;
		double imin,imax;
		
		q1->getFieldRange(imin,imax);
		cout<< q1->getName() << " field range " << imin << " " << imax << endl;
		
		q1->getStrengthRange(imin,imax,2.0);
		cout<< q1->getName() << " strength range@2.0GeV " << imin << " " << imax << endl;
		
		q1->getExcitationRange(imin,imax);
		cout<< q1->getName() << " current range " << imin << " " << imax << endl;
		
		int steps=100;
		double delta=(imax-imin)/steps;
		double mfield,strength,mlen,curr;
		cout << "#curr\tmfield\tstrength\tmlen" << endl;
		for(int i=0;i<=steps;i++){
			curr=imin+delta*i;
			mfield=q1->excitationToField(curr);
			mlen=q1->excitationToMagneticLength(curr);
			strength=q1->excitationToStrength(curr,2.0);
			cout << curr << "\t" << mfield << "\t" << strength << "\t"<< mlen << endl;
		}
		double st=q1->excitationToStrength(43.52617,2.0);
		cout << "at nom curr (43.52617) for 2.0GeV- field: " << q1->excitationToField(43.52617) << " strength:" << st << endl;
		
		cout << q1->getName() << " " << q1->getPsName() << " nom curr at 2GeV (inverse calc) " << q1->strengthToExcitation(st,2.0) << endl;
		
		
		
		if(q1) delete q1;
	}
	catch (std::range_error& ex){
		cout << "test building of Sextupole.. ...FAIL,  range_error: " << ex.what() << endl;
	}
	catch (std::invalid_argument& ex){
		cout << "test building of Sextupole.. ...FAIL,  invalid_argument: " << ex.what() << endl;
	}
	catch(...){
		cout << "test building of Sextupole.. ...FAIL,  unexpected exception!"  << endl;
	}
	*/
	return 0;
}
