#include <iostream>
#include "solenoid.h"

#include <multipolynomial.h>
#include <spline.h>
#include "magnetfactory.h"
#include <stdexcept>

using namespace std;
using namespace magnetfield;
using namespace Interpolator;

int main(){
	cout << "test of Field::Solenoid classes" << endl;
	
	InterpolatingPolynomial p0,p1,p2;
	p0.xMin=0.5;
	p0.xMax=1.5;
	p0.coefficient.push_back(1.1);
	
	p1.xMin=1.5;
	p1.xMax=2.5;
	p1.coefficient.push_back(1.1);
	p1.coefficient.push_back(2.2);
	
	p2.xMin=2.5;
	p2.xMax=3.5;
	p2.coefficient.push_back(1.1);
	p2.coefficient.push_back(2.2);
	p2.coefficient.push_back(3.3);
	
	//stuff polynomial into vector
	
	InterpolatingPolynoamialVector pvector;
	pvector.push_back(p0);
	pvector.push_back(p1);
	pvector.push_back(p2);
	
	multiPolynomial *pfield = new multiPolynomial(pvector);
	
	doubleVector x,y;
	const int NP=30;
	for(int i=0; i < NP; i++){
		x.push_back(i*.02);
		y.push_back(i*i*.02);
	}
	Spline* invfield = new Spline(x,y);
	
	Interpolator::Interpolator *field,*inverse,*len;
	
	
	field=pfield;
	inverse=invfield;
	len=pfield;
	bool focus=true;
 	Solenoid*myq= new Solenoid(focus,*field,*inverse,*len,"test","pstest");
	
	double pv=pfield->evaluate(1.1);
	delete pfield; pfield=nullptr;
	delete invfield; invfield=nullptr;
	double currm,currM;
	myq->getExcitationRange(currm,currM);
	
	cout << "min I:" << currm << " max I:" << currM << endl;
	double v=myq->excitationToField((currm+currM)/2.0);
	
	double gmin,gmax;
	myq->getFieldRange(gmin,gmax);
	cout << "min g:" << gmin << " max g:" << gmax << endl;
	double ex=myq->fieldToExcitation(gmin);
	double le=myq->excitationToMagneticLength((currm+currM)/2.0);
	cout << v << " " << pv << " " << ex << " " << le << endl;
	
	{
		cout << "test strength ( )...";
		try {
			double k=myq->excitationToStrength((currm+currM)/2.0, 1.0);
			cout << "k: " << k;
			k=myq->excitationToStrength((currm+currM)/2.0, 0.0);
			cout << " ...FAIL,  NO Exception !"  << endl;
		}
		catch (std::range_error& e){
			cout << " ...OK,  range error: " << e.what() << endl;
		}
		catch(...) {
			cout << " ...FAIL,  unexpected exception!"  << endl;
		}
	
	}
	
	{
		cout << "test excitation(field)...";
		try {
			double f=myq->fieldToExcitation(gmin);
			cout << "f: " << f << " gmin:" <<gmin;
			myq->getStrengthRange(gmin,gmax,2.0);
			f=myq->strengthToExcitation(gmin, 2.0);
			cout << " f[2.0GeV]: " << f << " gmin[2.0GeV]:" << gmin;
			f=myq->strengthToExcitation(gmin, 0.0);
			cout << " ...FAIL,  NO Exception !"  << endl;
		}
		catch (std::range_error& e){
			cout << " ...OK,  range error: " << e.what() << endl;
		}
		catch(...) {
			cout << " ...FAIL,  unexpected exception!"  << endl;
		}
		
	}
	
	
  	delete myq;
	
	
	
	//test building of solenoid
	
	try{
		Solenoid*sol1=nullptr;
		cout << "test building of Solenoid.. " ;
		Magnetfactory* mf1 =new Magnetfactory();
		sol1=mf1->newSolenoid("SOL_INJ.01");
		delete mf1;
		cout << " . OK! " << endl;
	
	
	//play wtih sol1;
		double imin,imax;
	
		sol1->getFieldRange(imin,imax);
		cout<< sol1->getName() << " field range " << imin << " " << imax << endl;
	
		sol1->getStrengthRange(imin,imax,5.0e-3);
		cout<< sol1->getName() << " strength range@5MeV " << imin << " " << imax << endl;
	
		sol1->getExcitationRange(imin,imax);
		cout<< sol1->getName() << " current range " << imin << " " << imax << endl;
	
		int steps=100;
		double delta=(imax-imin)/steps;
		double mfield,strength,mlen,curr;
		cout << "#curr\tmfield\tstrength\tmlen" << endl;
		for(int i=0;i<=steps;i++){
			curr=imin+delta*i;
			mfield=sol1->excitationToField(curr);
			mlen=sol1->excitationToMagneticLength(curr);
			strength=sol1->excitationToStrength(curr,5.0e-3);
			cout << curr << "\t" << mfield << "\t" << strength << "\t"<< mlen << endl;
		}
		double st=sol1->excitationToStrength(33.0,5.0e-3);
		cout << "SOL_INJ.01 at nom curr (33.0) for 5MeV- field: " << sol1->excitationToField(33.0) << " strength:" << st << endl;
	
		cout << sol1->getName() << " " << sol1->getPsName() << "SOL_INJ.01 nom curr at 5MeV (inverse calc) " << sol1->strengthToExcitation(st,5.0e-3) << endl;
	
	
	
		if(sol1) delete sol1;
	}
	catch (std::range_error& e){
			cout << "test building of Solenoid SOL_INJ.01.. ...FAIL,  range_error: " << e.what() << endl;
		}
 	catch (std::invalid_argument& e){
 		cout << "test building of Solenoid SOL_INJ.01.. ...FAIL,  invalid_argument: " << e.what() << endl;
 	}
 	catch(...){
 		cout << "test building of Solenoid SOL_INJ.01.. ...FAIL,  unexpected exception!"  << endl;
 	}
	
	return 0;
}
